C ********************************************************************
C * BANDPLOT - Plotting and interpolation of energy bands            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  BPFRMI                                                          *
C *                                                                  *
C *  Routine of BANDPLOT                                             *
C *                                                                  *
C *  Usage......: Marks fermi energy.                                *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs) c/o                          *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: 15.04.92                                           *
C *  Revision...: 18.06.97/mbs                                       *
C *  Remarks....: Identical with DPFRMI.                             *
C *               Changed for use with PGPLOT library.               *
C *                                                                  *
C *  Called from: BPGRPH                                             *
C *                                                                  *
C *  Calls......: BPTRAN,BPTRAP                                      *
C *               PGPLOT routines PGSWIN,PGSVP,PGSLS,PGMOVE,PGDRAW   *
C *                                                                  *
C ********************************************************************

      SUBROUTINE BPFRMI(VPXOFF,VPYOFF,VPXFUL,VPYFUL,VPXAXW,VPYAXH,
     .                  VPXEND,VPYEND,EMIN,EMAX,EFERMI,FFLINE)
C
C NAG/GKS version.
C
C      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C PGPLOT version.
C
      IMPLICIT REAL(A-H,O-Z)

      LOGICAL      FFLINE
      CHARACTER    BSL
      CHARACTER*80 STRING

      PARAMETER (ILEFT=0,MARGIN=0,BSL='\\')

C
C Set datarange and viewport.
C
C NAG/GKS version.
C
C      CALL J06WBF(0.0,VPXFUL,0.0,VPYFUL,MARGIN)
C      CALL J06WCF(0.0,VPXFUL,0.0,VPYFUL)
C
C PGPLOT version.
C
      CALL PGSWIN(0.0,VPXFUL,0.0,VPYFUL)
      CALL PGSVP(0.0,VPXFUL,0.0,VPYFUL)  
C
C Transform fermi energy coordinate into NDC.
C
      PENX=VPXOFF-0.05
      PENY=(EFERMI-EMIN)/(EMAX-EMIN)*VPYAXH+VPYOFF-0.006

      STRING=BSL//'epsilon '//BSL//'sub f '//BSL//'endsub '//BSL//
     .       'arrowrgt'
C
C NAG/GKS version.
C
C      CALL BPTRAN(STRING,PENX,PENY,IRIGHT)
C
C PGPLOT version.
C
      CALL BPTRAP(STRING,PENX,PENY,0.0,IRIGHT)

      IF (FFLINE) THEN
C
C Set datarange and viewport.
C
C NAG/GKS version.
C
C        CALL J06WBF(0.0,VPXAXW,EMIN,EMAX,MARGIN)
C        CALL J06WCF(VPXOFF,VPXEND,VPYOFF,VPYEND)
C
C PGPLOT version.
C
        CALL PGSWIN(0.0,VPXAXW,EMIN,EMAX)
        CALL PGSVP(VPXOFF,VPXEND,VPYOFF,VPYEND)
C
C Set current linestyle.
C
C NAG/GKS version.
C
C        CALL J06YRF(2)
C
C PGPLOT version.
C
        CALL PGSLS(2)        
C
C Draw line.
C
C NAG/GKS version.
C
C        CALL J06YAF(0.0,EFERMI)
C        CALL J06YDF(VPXAXW,0.0)
C
C PGPLOT version.
C
        CALL PGMOVE(0.0,EFERMI)
        CALL PGDRAW(VPXEND,EFERMI)
C
C Set current linestyle.
C
C
C NAG/GKS version.
C
C        CALL J06YRF(1)
C
C PGPLOT version.
C
        CALL PGSLS(1)
      ENDIF
C
C NAG/GKS version.
C
C      CALL GUWK(1,1)
      RETURN
      END