C ********************************************************************
C * BANDPLOT - Plotting and interpolation of energy bands            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  EREAD                                                           *
C *                                                                  *
C *  Routine of BFIT (BANDPLOT)                                      *
C *                                                                  *
C *  Usage......: Reads in energies of a symmetry line ILABEL out of *
C *               a .EHE file. Uses KPOINT as index for k points.    *
C *               The right order of the k points has been given by  *
C *               the .BINPUT file made by BANDPL.                   *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs)                              *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for Inorg.+Phys. Chemistry               *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *  Date.......: 28.04.93                                           *
C *  Revision...: 01.07.97/mbs                                       *
C *  Remarks....: Replaces KSORT.                                    *
C *                                                                  *
C *  Called from: BFIT                                               *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE EREAD(KIN,EIN,KPOINT,NBTOT,NNOFIT,NBFIT,NK,ILABEL,
     .                 NSYMME,IUNIT)

C 
C NAG/GKS version.
C 
C      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
C
C PGPLOT version.
C
      IMPLICIT REAL (A-H,O-Z)
      INCLUDE 'sizes'
C
C E:   Energies
C DK:  Energy derivatives
C EIN: Energy buffer: Lower MAXK holds energies, higher MAXK holds
C      k weighted energy derivatives.
C NK:  Number of k points per actual symmetry line ISYM
C KIN: Ratios of k points over end point of actual symmetry line ISYM
C KC1: k points of actual symmetry line
C
      DIMENSION IDUMM1(MXSYME),KPOINT(MAXK)
      DIMENSION E(MAXBA),DK(3,MAXBA),EIN(2*MAXK,MAXBA),P(3)
C
C NAG/GKS version.
C
C      DOUBLE PRECISION KL(3),KD(3),KIN(MAXK),KC1(MAXK,3)
C
C PGPLOT version.
C
      REAL      KL(3),KD(3),KIN(MAXK),KC1(MAXK,3)
      CHARACTER CDUMMY*80
 
      DATA S1/0.003/
 
      DIV=1.0
      IF (NSYMME .GT. 0) THEN
        DIV=2.0
      ENDIF
 
      ILOW=NNOFIT+1
      IHI=ILOW+NBFIT-1
      IF (NNOFIT .EQ. 1) ILOW=ILOW-1
C
C Read in last k point of symmetry line
C
      REWIND IUNIT
      IDIFF=KPOINT(NK)-1
      IF (IDIFF .GT. 0) THEN
        DO I=1,IDIFF
          DO J=1,NBTOT+4
            READ(IUNIT,'(A80)') CDUMMY
          END DO
        END DO
      ENDIF
      READ(IUNIT,'(3F10.7)') (P(I),I=1,3)
      KOLD=0
      REWIND IUNIT
 
      WRITE(6,'(/,'' k points of symmetry line: '',I2)') ILABEL
      DO K=1,NK
        IDIFF=KPOINT(K)-KOLD-1
        IF (IDIFF .LT. 0) THEN
          REWIND IUNIT
          IDIFF=KPOINT(K)-1
        ENDIF
        IF (IDIFF .GT. 0) THEN
          DO I=1,IDIFF
            DO J=1,NBTOT+4
              READ(IUNIT,'(A80)') CDUMMY
            END DO
          END DO
        ENDIF
        READ(IUNIT,'(3F10.7)') (KC1(K,I),I=1,3)
        WRITE(6,'('' k point '',I3,'': '',3F15.10)') K,(KC1(K,I),I=1,3)
        IF (K .EQ. 1) THEN
          DO I=1,3
            KL(I)=KC1(K,I)
            P(I)=(P(I)-KL(I))*DIV
          END DO
        ENDIF
        DO I=1,3
          READ(IUNIT,'(A80)') CDUMMY
        END DO
        DO I=1,NBTOT
          READ(IUNIT,'(4F10.5,I4,10I3)') E(I),(DK(J,I),J=1,3),IDUMMY,
     .                                   (IDUMM1(J),J=1,MXSYME)
        END DO
 
        DO I=1,3
          KD(I)=KC1(K,I)-KL(I)
          IF (ABS(P(I)) .GE. S1) THEN
            RATIO=0.5*KD(I)/P(I)
          ENDIF
        END DO
        KIN(K)=ABS(RATIO)
 
        I1=1
        K2=K+NK
        DO I=ILOW,IHI
          EIN(K,I1)=E(I)
          EIN(K2,I1)=0.0
          DO J=1,3
            EIN(K2,I1)=EIN(K2,I1)+DK(J,I)*P(J)*2.0
          END DO
          I1=I1+1
        END DO
        KOLD=KPOINT(K)
      END DO
 
      RETURN
      END