C ********************************************************************
C * CEDiT - Crystal Electronic Dipole-induced Transitions            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  SUBROUTINE LOVLAP                                               *
C *                                                                  *
C *  Routine of EDiT/CEDiT                                           *
C *  version for IBM-PC MS-DOS, Microsoft FORTRAN and                *
C *              IBM-RISC UNIX, AIX XL FORTRAN                       *
C *                                                                  *
C *  Usage......: Calculates the overlaps.                           *
C *                                                                  *
C *  Authors....: unknown                                            *
C *                                                                  *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: -                                                  *
C *  Revision...: 29. 11. 1992 Ruedi Rytz                            *
C *                                                                  *
C *  Remarks....: Recursion formula to calculate the binominal       *
C *               coefficients added.                                *
C *                                                                  *
C *  Called from: DOPELE                                             *
C *               DOSELE                                             *
C *               PODELE                                             *
C *               POPELE                                             *
C *               POSELE                                             *
C *               SODELE                                             *
C *               SOPELE                                             *
C *               SOSELE                                             *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************

      SUBROUTINE LOVLAP(STRAD, A, B, SK1, SK2, R, L1, L2,
     .                  M1, N1, N2)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

        DOUBLE PRECISION STRAD, A(20), B(20), SK1, SK2, R
        INTEGER          L1, L2, M1, N1, N2

C VAR
      INTEGER          I, I1, J
      DOUBLE PRECISION BINCOE(9,9)
      LOGICAL          JGO, FIRST
      DIMENSION        FACT(25)

      SAVE FIRST, BINCOE
      DATA JGO/.FALSE./ FIRST/.TRUE./

C BEGIN

C Initialisierung der Binomialkoeffizienten

      IF (FIRST) THEN
        FIRST = .FALSE.

        DO 1 I = 1, 9
          BINCOE(I,1) = 1.0D0
          BINCOE(I,I) = 1.0D0
1       CONTINUE

        DO 2 I = 1, 8
          I1 = I - 1
          DO 2 J = 1, I1
            BINCOE(I+1,J+1) = BINCOE(I,J+1) + BINCOE(I,J)
2       CONTINUE

      ENDIF

C Ende der Initialisierung der Binomialkoeffizienten

      IF (JGO) GOTO 10
      JGO=.TRUE.
      FACT(1)=1.D0
      DO 5 I=2,25
5       FACT(I)=FACT(I-1)*DBLE(I-1)
10    CONTINUE
      M2=M1
      STRAD=0.D0
      RHOA=R*SK1
      RHOB=R*SK2
      TERMA=0.5D0**(L1+L2+1) * DSQRT(DBLE((L1+L1+1)*(L2+L2+1))*
     . FACT(L1-M1+1)*FACT(L2-M1+1)/(FACT(N1+N1+1)*FACT(N2+N2+1)*
     . FACT(L1+M1+1)*FACT(L2+M1+1))*RHOA**(N1+N1+1)*RHOB**(N2+N2+1))
      JEND=1+((L1-M1)/2)
      KEND=1+((L2-M2)/2)
      IEB=M1+1
      DO 50 J=1,JEND
        JU=J-1
        IAB=N1-L1+JU+JU+1
        ICB=L1-M1-JU-JU+1
        CON1=FACT(L1+L1-JU-JU+1)/(FACT(L1-M1-JU-JU+1)*FACT(JU+1)*
     .  FACT(L1-JU+1))
        DO 50 K=1,KEND
          KU=K-1
          CON12=CON1*FACT(L2+L2-KU-KU+1)/(FACT(L2-M2-KU-KU+1)*
     .    FACT(KU+1)*FACT(L2-KU+1))
          IEV=JU+KU+L2
          IF (2*(IEV/2).NE.IEV) CON12=-CON12
          IBB=N2-L2+KU+KU+1
          IDB=L2-M2-KU-KU+1
          VALUE=0.D0
          DO 90 I6=1,IEB
            DO 90 I5=1,IEB
              VALUE1=BINCOE(IEB,I6)*BINCOE(IEB,I5)
              IEV=I5+I6
              IF (2*(IEV/2).NE.IEV) VALUE1=-VALUE1
              DO 90 I4=1,IDB
                VALUE1=-VALUE1
                VALUE2=BINCOE(IDB,I4)*VALUE1
                DO 90 I3=1,ICB
                  VALUE3=BINCOE(ICB,I3)*VALUE2
                  DO 90 I2=1,IBB
                    VALUE3=-VALUE3
                    VALUE4=BINCOE(IBB,I2)*VALUE3
                    DO 90 I1=1,IAB
                      TERM=VALUE4*BINCOE(IAB,I1)
                      IR=I1+I2+IEB+IEB-I6-I6-I3+IDB-I4+ICB-1
                      IP=IAB-I1+IBB-I2+IEB+IEB-I5-I5+ICB-I3+IDB-I4+1
90                    VALUE=VALUE+A(IP)*B(IR)*TERM
50      STRAD=STRAD+VALUE*CON12
      STRAD=STRAD*TERMA

      RETURN
      END
C END
