C ********************************************************************
C * CEDiT - Crystal Electronic Dipole-induced Transitions            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  SUBROUTINE SOPELE                                               *
C *                                                                  *
C *  Routine of EDiT/CEDiT                                           *
C *  version for IBM-PC MS-DOS, Microsoft FORTRAN and                *
C *              IBM-RISC UNIX, AIX XL FORTRAN                       *
C *                                                                  *
C *  Usage......: Calculates the <s|O|p> transition elements of the  *
C *               transition matrix.                                 *
C *                                                                  *
C *  Authors....: Ruedi Rytz                                         *
C *                                                                  *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: July 1992                                          *
C *  Revision...: -                                                  *
C *                                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: OBDIAG                                             *
C *                                                                  *
C *  Calls......: ABFNS                                              *
C *               LOVLAP                                             *
C *               SPOVL                                              *
C *               PPOVL                                              *
C *                                                                  *
C ********************************************************************

      SUBROUTINE SOPELE (M, HQZ, N2, SK1, SK2, R,
     .                   PTR, LDIST, POL, CURRJ)

      IMPLICIT NONE

        DOUBLE PRECISION  M
        INTEGER           HQZ, N2
        DOUBLE PRECISION  SK1, SK2, R, PTR(9), LDIST
        CHARACTER*1       POL
        INTEGER           CURRJ

C VAR
      INTEGER          L1, L2, ML, MAX, N1
      DOUBLE PRECISION A(20), B(20), PPOP, PPOS, PPO(3,3), SPOS, SPO(3) 
      DOUBLE PRECISION NORM

      INCLUDE 'snorm.inc'

C BEGIN

C Berechne zuerst den "normalen" s-p-Overlap

      N1 = HQZ

      L1 = 0
      L2 = 1
      ML = 0
      MAX = N1 + N2

      CALL ABFNS(A, B, SK1, SK2, R, MAX)
      CALL LOVLAP(SPOS, A, B, SK1, SK2, R, L1, L2,
     .            ML, N1, N2)

      CALL SPOVL(SPO, SPOS, PTR)

C Berechne nun den p-p-Overlap nachem man den Operator auf das s-Orbital
C angewandt hat.

      N1 = N1 + 1
      L1 = 1
      MAX = N1 + N2

      CALL ABFNS(A, B, SK1, SK2, R, MAX)
      CALL LOVLAP(PPOS, A, B, SK1, SK2, R, L1, L2,
     .            ML, N1, N2)

      PPOS = -PPOS

      ML = 1

      CALL LOVLAP(PPOP, A, B, SK1, SK2, R, L1, L2,
     .            ML, N1, N2)

      CALL PPOVL(PPO, PPOS, PPOP, PTR)

C Ende Berchnung des p-p-Overlaps

      IF (POL .EQ. 'X') THEN
          M = M * (NORM * PPO(1,CURRJ-1)
     .          + LDIST * SPO(CURRJ-1) )
      ELSE IF (POL .EQ. 'Y') THEN
        M = M * (NORM * PPO(2,CURRJ-1)
     .         + LDIST * SPO(CURRJ-1) )
      ELSE
        M = M * (NORM * PPO(3,CURRJ-1)
     .         + LDIST * SPO(CURRJ-1) )
      ENDIF

      RETURN
      END
C END
