C ********************************************************************
C * DOSPLOT -  Plots density of states (DOS)                         *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  DOSPLOT                                                         *
C *                                                                  *
C *  Usage......: Plots density of states (DOS), integrated DOS,     *
C *               AO, FMO projections of the DOS and crystal orbital *
C *               overlap populations (COOP) generated with the      *
C *               extended-Hueckel properties calculations           *
C *               (EHPC) program.                                    *
C *               Used graphic library: PGPLOT.                      *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs) c/o                          *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: 28.04.92                                           *
C *  Revision...: 12.02.93/mbs, 18.10.93/mbs,  01.07.97/mbs          *
C *  Remarks....: Smoothed integrated DOS and COOP implemented.      *
C *               For OS specific changes look for "VMS", "AIX",     *
C *               "MS-Windows", "Windows NT".                        *
C *               Major revision for use with PGPLOT graphics        *
C *               subroutine library.                                *  
C *               Calls of NAG/GKS subroutines commented out.        *
C *                                                                  *
C *  Calls......: DPPLOT,DPGRPH,DPRDRW                               *
C *               PGPLOT routines PGOPEN,PGSLCT,PGASK,PGSCR,PGPAP,   *
C *               PGPAGE,PGCLOS,PGEND                                *
C *                                                                  *
C ********************************************************************
C
C MS-Windows version: Comment PROGRAM statement.
C VMS, AIX and Windows NT version: Uncomment PROGRAM statement.
C
      PROGRAM DOSPLT
C
C MS-Windows version: Uncomment SUBROUTINE statement.
C VMS and AIX version: Comment SUBROUTINE statement.
C
C      SUBROUTINE GKSAPP
C
C NAG/GKS version.
C
C      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
C
C PGPLOT version.
C
      IMPLICIT REAL(A-H,O-Z)

      INCLUDE 'sizes'

      DIMENSION    ENERGY(MXMESH),TEXTXY(MXTEXT,2),DOFFST(MAXBUF)
      DIMENSION    NCARDA(MXCARD),NCARDF(MXCARD),NCARDC(MXCARD)
      DIMENSION    ILTYPE(2*MAXBUF),ISHADE(2*MAXBUF),IBFNDX(2*MAXBUF)
      DIMENSION    TXTANG(MXTEXT),ITXPOS(MXTEXT) 
      CHARACTER*80 TITLA(MXCARD,MXTITL),TITLF(MXCARD,MXTITL),
     .             TITLC(MXCARD,MXTITL),CDOSTI,CCOPTI,CENETI,
     .             CTEXT(MXTEXT)
      CHARACTER*72 BUFTIT(2*MAXBUF),STRING
      CHARACTER    CANSWR
      LOGICAL      DPAPND,DPOLAY,FARBIT,FTEMP,FFLINE,
     .             FSELEC(MAXSEL),FARBTR(2*MAXBUF),FCLOSE(2*MAXBUF)

C
C NAG/GKS version.
C
C      REAL         WLINE
C
C PGPLOT version.
C
      INTEGER      PGOPEN
      LOGICAL FPGASK

      COMMON /BUF/BUFFER(2*MAXBUF*MXMESH),BUFMAX(2*MAXBUF),
     .            BUFMIN(2*MAXBUF)

C
C Reset graph options.
C
      PARAMETER(MAXBU2=2*MAXBUF)

      DATA FSELEC/MAXSEL*.FALSE./,FARBTR/MAXBU2*.FALSE./,
     .     FCLOSE/MAXBU2*.FALSE./,ILTYPE/MAXBU2*1/,ISHADE/MAXBU2*0/,
     .     DOFFST/MAXBUF*0/

      DATA FPGASK/.FALSE./
C
C Vertical plotting.
C
      VPXFUL=VVXFUL
      VPYFUL=VVYFUL
      VPXAXW=VVXAXW
      VPYAXH=VVYAXH
      VPXOFF=VVXOFF
      VPYOFF=VVYOFF
      VPXDOO=VPXOFF
      VPXCOO=VPXOFF
      VPXDOW=VPXAXW
      VPXCOW=VPXAXW

      DOSMN=1000.
      DOSMX=0.0
      COOPMX=0.0
      XRANGE=1.0
      IBUFDS=0
      IBUFCP=0
      ITXT=0
C
C Unit 1: File with interpolated DOS, ...
C
C MS-Windows version: Add FILE='fort.1'
C VMS and AIX version: Remove FILE='fort.1'
C
      OPEN(UNIT=1,STATUS='OLD',ERR=890,FILE='fort.1')
C      OPEN(UNIT=1,STATUS='OLD',ERR=890)
      READ(1,20) NPRO,NPROJA,NPROJF,NPROJC,NMESH
      READ(1,20) (NCARDA(I),I=1,MXCARD)
      READ(1,20) (NCARDF(I),I=1,MXCARD)
      READ(1,20) (NCARDC(I),I=1,MXCARD)
   20 FORMAT(5I4)
      READ(1,30) ((TITLA(I,J),J=1,MXTITL),I=1,MXCARD)
      READ(1,30) ((TITLF(I,J),J=1,MXTITL),I=1,MXCARD)
      READ(1,30) ((TITLC(I,J),J=1,MXTITL),I=1,MXCARD)
   30 FORMAT(A80)
      DO I=1,NMESH
        READ(1,'(F12.5)') ENERGY(I)
      END DO
      EMIN=ENERGY(1)
      EMAX=ENERGY(NMESH)
C
C Initialize plotinterface.
C
C NAG/GKS version.
C
C      CALL NAGGKS(10)
C      CALL J06WAF
C
C PGPLOT version.
C
      IDEVSC=PGOPEN(WDEVIC)
      CALL PGSLCT(IDEVSC)
      CALL PGASK(FPGASK)
      CALL PGSCR(0,1.,1.,1.)
      CALL PGSCR(1,0.,0.,0.)
      CALL PGPAP(7.5,1.0)

      WRITE(6,10)
   10 FORMAT(//,' *************************************************',/,
     .          ' *                  D O S P L O T                *',/,
     .          ' *             Release September 1997            *',/,
     .          ' * --------------------------------------------- *',/,
     .          ' *      Plotting of density of states (DOS),     *',/,
     .          ' *    AO and FMO contributions to the DOS, and   *',/,
     .          ' *   crystal orbital overlap populations (COOP)  *',/,
     .          ' *                 Martin Braendle               *',/,
     .          ' *   Department of Chemistry and Biochemistry    *',/,
     .          ' *        University of Bern, Switzerland        *',/,
     .          ' *                                               *',/,
     .          ' *          (C) 1997 by Gion Calzaferri          *',/,
     .          ' *   DOSPLOT comes with absolutely no warranty.  *',/,
     .          ' * This is free software and you are welcome to  *',/,
     .          ' *   redistribute it under certain conditions.   *',/,
     .          ' *  For details see GNU General Public License.  *',/,
     .          ' *************************************************',/,
     .2X)

C
C AIX GKSCO (PHIGS) version: Uncomment following three lines.
C VMS, MS-Windows, and Windows NT version: Comment following three lines.
C
C      CALL GSDS(1,0,1)
C      CALL GSCR(1,0,1.0,1.0,1.0)
C      CALL GSCR(1,1,0.0,0.0,0.0)
C
C Fill brushes are GKS pattern style. Chosen patterns are device
C dependent and must be changed for other implementations. Brush
C assign start with brush number 2, because for unknown reasons
C shade area factor of brush number 1 is changed unpredictably if
C text is used.
C
C If PGPLOT version, comment next 10 lines.
C      ISHSEL=0
C      CALL J06XJF(ISHSEL,.TRUE.,.FALSE.)
C      IPALET=1
C      ICOLOR=1
C      IFSTYL=2
C      CALL J06XKF(2,IPALET,ICOLOR,IFSTYL,1)
C      DO 60 IBRUSH=3,16
C        CALL J06XKF(IBRUSH,IPALET,ICOLOR,IFSTYL,IBRUSH+11)
C   60 CONTINUE
C
C Set line width scale factor.
C
C PGPLOT version: Comment next 2 lines
C      WLINE=3.0
C      CALL GSLWSC(WLINE)
C
C Main menu parameters.
C
      IMDOS=-1
      IMIDOS=-1
      IMTOTE=-1
      IMAO=-1
      IMIAO=-1
      IMFMO=-1
      IMIFMO=-1
      IMCOOP=-1
      IMICOP=-1
      IMOUTP=-1

   70 IMENU=0
      WRITE(6,'(/,'' Main menu:'',/)')
      IF (NPRO .NE. 0) THEN
        IMENU=IMENU+1
        IMDOS=IMENU
        WRITE(6,'('' ('',I2,'') Density of states'')') IMDOS
        IMENU=IMENU+1
        IMIDOS=IMENU
        WRITE(6,'('' ('',I2,'') Integrated density of states'')') IMIDOS
        IMENU=IMENU+1
        IMTOTE=IMENU
        WRITE(6,'('' ('',I2,'') Total energy'')') IMTOTE
      ENDIF
      IF (NPROJA .NE. 0) THEN
        IMENU=IMENU+1
        IMAO=IMENU
        WRITE(6,'('' ('',I2,'') AO contribution to DOS ('',I1,
     .        '' projection(s))'')') IMAO,NPROJA
        IMENU=IMENU+1
        IMIAO=IMENU
        WRITE(6,'('' ('',I2,'') Integrated AO contribution to DOS ('',
     .        I1,'' projection(s))'')') IMIAO,NPROJA
      ENDIF
      IF (NPROJF .NE. 0) THEN
        IMENU=IMENU+1
        IMFMO=IMENU
        WRITE(6,'('' ('',I2,'') FMO contribution to DOS ('',I1,
     .        '' projection(s))'')') IMFMO,NPROJF
        IMENU=IMENU+1
        IMIFMO=IMENU
        WRITE(6,'('' ('',I2,'') Integrated FMO contribution to DOS ('',
     .        I1,'' projection(s))'')') IMIFMO,NPROJF
      ENDIF
      IF (NPROJC .NE. 0) THEN
        IMENU=IMENU+1
        IMCOOP=IMENU
        WRITE(6,'('' ('',I2,'') COOP ('',I1,'' projection(s))'')') 
     .        IMCOOP,NPROJC
        IMENU=IMENU+1
        IMICOP=IMENU
        WRITE(6,'('' ('',I2,'') Integrated COOP ('',I1,
     .        '' projection(s))'')') IMICOP,NPROJC
      ENDIF
      IMENU=IMENU+1
      IMGRPH=IMENU
      WRITE(6,'(/,'' ('',I2,'') Modify graph'')') IMGRPH
      IF (IBUFDS+IBUFCP .GT. 0) THEN
        IMENU=IMENU+1
        IMDELE=IMENU
        WRITE(6,'('' ('',I2,'') Delete buffer'')') IMDELE
      ENDIF
C
C AIX version: Comment next four program lines.
C VMS and MS-Windows version: Uncomment next four program lines.
C
      IMENU=IMENU+1
      IMOUTP=IMENU
      WRITE(6,'('' ('',I2,'') Generate outputfile'')') IMOUTP
      IMENU=IMENU+1
      IMSAVE=IMENU
      WRITE(6,'('' ('',I2,'') Save graph layout'')') IMSAVE
      IMENU=IMENU+1
      IMLOAD=IMENU
      WRITE(6,'('' ('',I2,'') Load graph layout'')') IMLOAD
      WRITE(6,'(/,'' ( 0) Quit program'',/)')

  210 WRITE(6,'('' Selection: '',$)')
      READ(5,'(I2)',ERR=210) ISEL
      IF ((ISEL .LT. 0) .OR. (ISEL .GT. IMENU)) GOTO 210
C
C ---------------------------------------------------------------------
C DOS, integrated DOS and total energy.
C
      IF ((ISEL .GE. IMDOS) .AND. (ISEL .LE. IMTOTE)) THEN
        IFNDX2=ISEL
        IF (ISEL .EQ. 1) THEN
          STRING='Density of states'
        ELSE IF (ISEL .EQ. 2) THEN
          STRING='Integrated density of states'
        ELSE
          STRING='Total energy'
        ENDIF
C
C Entry point for all DOS plots.
C
  240   IF ((IBUFCP .GT. 0) .AND. (IBUFDS .EQ. 0)) THEN
          IF (DPAPND(IDUMMY)) THEN
            VPXCOO=VPXOFF
            VPXCOW=SCCOOP*VPXAXW
            VPXDOO=VPXOFF+VPXCOW
            VPXDOW=SCDOS*VPXAXW
C
C Redraw COOP plot with new dimensions.
C
            IFINDX=0
            IPLTTP=2
            FARBIT=.FALSE.
            XMAX=0.0
C
C Clear output device
C NAG/GKS version.
C
C            CALL J06WDF
C
C PGPLOT version.
C
            CALL PGPAGE
            CALL DPPLOT(VPXCOO,VPXCOW,VPYOFF,VPYAXH,EMIN,EMAX,ENERGY,
     .                  XMAX,XRANGE,DOSMN,DOSMX,COOPMX,DOFFST,
     .                  NMESH,IFINDX,IBUFDS,IBUFCP,IPLTTP,ILTYPE,ISHADE,
     .                  IBFNDX,STRING,BUFTIT,FARBTR,FCLOSE,FARBIT)
          ELSE
C
C New plot. Clear output device.
C NAG/GKS version.
C
C            CALL J06WDF
C
C PGPLOT version.
C
            CALL PGPAGE
            IBUFCP=0
            COOPMX=0.0
            XMAX=0.0
            DOSMX=0.0
            DOSMN=1000.
            VPXDOO=VPXOFF
            VPXDOW=VPXAXW
            DO I=1,MAXBUF
              DOFFST(I)=0.0
            END DO
            DO I=1,2*MAXBUF
              FARBTR(I)=.FALSE.
              FCLOSE(I)=.FALSE.
              ISHADE(I)=0
              ILTYPE(I)=1
            END DO
          ENDIF
        ENDIF

        IF (IBUFDS .GT. 0) THEN
          FTEMP=DPOLAY(FARBIT)
          IF ( .NOT. FTEMP) THEN
            IBUFDS=0
            IBUFCP=0
            XMAX=0.0
            COOPMX=0.0
            DOSMX=0.0
            DOSMN=1000.
            VPXDOO=VPXOFF
            VPXDOW=VPXAXW
            DO I=1,MAXBUF
              DOFFST(I)=0.0
            END DO
            DO I=1,2*MAXBUF
              FARBTR(I)=.FALSE.
              FCLOSE(I)=.FALSE.
              ISHADE(I)=0
              ILTYPE(I)=1
            END DO
C
C Clear output device
C NAG/GKS version.
C
C            CALL J06WDF
C
C PGPLOT version.
C
            CALL PGPAGE
          ENDIF
C
C NAG/GKS version.
C
C          IF (FTEMP .AND. ( .NOT. FARBIT)) CALL J06WDF
C
C PGPLOT version.
C
          IF (FTEMP .AND. ( .NOT. FARBIT)) CALL PGPAGE
        ENDIF
C
C Draw or append DOS plot.
C
        IPLTTP=1
        CALL DPPLOT(VPXDOO,VPXDOW,VPYOFF,VPYAXH,EMIN,EMAX,ENERGY,XMAX,
     .              XRANGE,DOSMN,DOSMX,COOPMX,DOFFST,
     .              NMESH,IFNDX2,IBUFDS,IBUFCP,IPLTTP,ILTYPE,ISHADE,
     .              IBFNDX,STRING,BUFTIT,FARBTR,FCLOSE,FARBIT)
C
C ---------------------------------------------------------------------
C AO projection of DOS.
C
      ELSE IF ((ISEL .EQ. IMAO) .OR. (ISEL .EQ. IMIAO)) THEN
        WRITE(6,'(/,'' AO projection(s) of DOS:'',/)')
        DO I=1,NPROJA
          DO J=1,NCARDA(I)
            IF (J .EQ. 1) THEN
              WRITE(6,'('' ('',I1,'') '',A75)') I,TITLA(I,J)
            ELSE
              WRITE(6,'(''     '',A75)') TITLA(I,J)
            ENDIF
          END DO
        END DO
        WRITE(6,'('' '')')

  330   WRITE(6,'('' Selection: '',$)')
        READ(5,'(I1)',ERR=330) ISELAO
        IF ((ISELAO .LT. 1) .OR. (ISELAO .GT. NPROJA)) GOTO 330
        STRING='AOPROJ  '//TITLA(ISELAO,1)(1:64)
C
C Not at all structured programming, but much coding can be avoided.
C
        IFNDX2=NPRO+2*ISELAO-1
        XMAX=0.0
        IF (ISEL .EQ. IMIAO) THEN
          IFNDX2=IFNDX2+1
  340     WRITE(6,'('' Enter maximum of AO projected DOS curve: '',$)')
          READ(5,'(F10.5)',ERR=340) XMAX
        ENDIF
        GOTO 240
C
C ---------------------------------------------------------------------
C FMO projection of DOS.
C
      ELSE IF ((ISEL .EQ. IMFMO) .OR. (ISEL .EQ. IMIFMO)) THEN
        WRITE(6,'(/,'' FMO projection(s) of DOS:'',/)')
        DO I=1,NPROJF
          DO J=1,NCARDF(I)
            IF (J .EQ. 1) THEN
              WRITE(6,'('' ('',I1,'') '',A75)') I,TITLF(I,J)
            ELSE
              WRITE(6,'(''     '',A75)') TITLA(I,J)
            ENDIF
          END DO
        END DO
        WRITE(6,'('' '')')

  410   WRITE(6,'('' Selection: '',$)')
        READ(5,'(I1)',ERR=410) ISELFM
        IF ((ISELFM .LT. 1) .OR. (ISELFM .GT. NPROJF)) GOTO 410
        STRING='FMOPROJ '//TITLF(ISELFM,1)(1:64)
C
C Not at all structured programming, but much coding can be avoided.
C
        IFNDX2=NPRO+2*NPROJA+2*ISELFM-1
        XMAX=0.0
        IF (ISEL .EQ. IMIFMO) THEN
          IFNDX2=IFNDX2+1
  420     WRITE(6,'('' Enter maximum of FMO projected DOS curve: '',$)')
          READ(5,'(F10.5)',ERR=420) XMAX
        ENDIF
        GOTO 240
C
C ---------------------------------------------------------------------
C COOP.
C
      ELSE IF ((ISEL .EQ. IMCOOP) .OR. (ISEL .EQ. IMICOP)) THEN
        WRITE(6,'(/,'' COOP between:'',/)')
        DO I=1,NPROJC
          DO J=1,NCARDC(I)
            IF (J .EQ. 1) THEN
              WRITE(6,'('' ('',I1,'') '',A75)') I,TITLC(I,J)
            ELSE
              WRITE(6,'(''     '',A75)') TITLC(I,J)
            ENDIF
          END DO
        END DO
        WRITE(6,'('' '')')
  500   WRITE(6,'('' Selection: '',$)')
        READ(5,'(I1)',ERR=500) ISELCP
        IF ((ISELCP .LT. 1) .OR. (ISELCP .GT. NPROJC)) GOTO 500
        STRING='OVLPROJ '//TITLC(ISELCP,1)(1:64)

        IFNDX2=NPRO+2*NPROJA+2*NPROJF+2*ISELCP-1
        IF (ISEL .EQ. IMICOP) IFNDX2=IFNDX2+1
        IF ((IBUFCP .EQ. 0) .AND. (IBUFDS .GT. 0)) THEN
          IF (DPAPND(IDUMMY)) THEN
            VPXDOO=VPXOFF
            VPXDOW=SCDOS*VPXAXW
            VPXCOO=VPXOFF+VPXDOW
            VPXCOW=SCCOOP*VPXAXW
C
C Redraw DOS plot with new dimensions.
C
            IFINDX=0
            IPLTTP=1
            XMAX=0.0
            FARBIT=.FALSE.
C
C NAG/GKS version.
C
C            CALL J06WDF
C
C PGPLOT version.
C
            CALL PGPAGE
            CALL DPPLOT(VPXDOO,VPXDOW,VPYOFF,VPYAXH,EMIN,EMAX,ENERGY,
     .                  XMAX,XRANGE,DOSMN,DOSMX,COOPMX,DOFFST,
     .                  NMESH,IFINDX,IBUFDS,IBUFCP,IPLTTP,ILTYPE,ISHADE,
     .                  IBFNDX,STRING,BUFTIT,FARBTR,FCLOSE,FARBIT)
          ELSE
C
C New plot.
C NAG/GKS version.
C
C            CALL J06WDF
C
C PGPLOT version.
C
            CALL PGPAGE
            IBUFDS=0
            COOPMX=0.0
            DOSMX=0.0
            DOSMN=1000.0
            VPXCOO=VPXOFF
            VPXCOW=VPXAXW
            DO I=1,MAXBUF
              DOFFST(I)=0.0
            END DO
            DO I=1,2*MAXBUF
              FARBTR(I)=.FALSE.
              FCLOSE(I)=.FALSE.
              ISHADE(I)=0
              ILTYPE(I)=1
            END DO
          ENDIF
        ENDIF

        IF (IBUFCP .GT. 0) THEN
          FTEMP=DPOLAY(FARBIT)
          IF ( .NOT. FTEMP) THEN
C
C New plot.
C
            IBUFDS=0
            IBUFCP=0
            COOPMX=0.0
            DOSMX=0.0
            DOSMN=1000.0
            VPXCOO=VPXOFF
            VPXCOW=VPXAXW
            DO I=1,MAXBUF
              DOFFST(I)=0.0
            END DO
            DO I=1,2*MAXBUF
              FARBTR(I)=.FALSE.
              FCLOSE(I)=.FALSE.
              ISHADE(I)=0
              ILTYPE(I)=1
            END DO
C
C NAG/GKS version.
C
C            CALL J06WDF
C
C PGPLOT version.
C
            CALL PGPAGE
          ENDIF
C
C NAG/GKS version
C
C          IF (FTEMP .AND. ( .NOT. FARBIT)) CALL J06WDF
C
C PGPLOT version.
C
          IF (FTEMP .AND. ( .NOT. FARBIT)) CALL PGPAGE
        ENDIF
C
C Draw or append COOP plot.
C
        IPLTTP=2
        XMAX=0.0
        CALL DPPLOT(VPXCOO,VPXCOW,VPYOFF,VPYAXH,EMIN,EMAX,ENERGY,
     .              XMAX,XRANGE,DOSMN,DOSMX,COOPMX,DOFFST,
     .              NMESH,IFNDX2,IBUFDS,IBUFCP,IPLTTP,ILTYPE,ISHADE,
     .              IBFNDX,STRING,BUFTIT,FARBTR,FCLOSE,FARBIT)
C
C ---------------------------------------------------------------------
C Change graph.
C
      ELSE IF ((ISEL .EQ. IMGRPH) .AND.
     .         ((IBUFDS .GT. 0) .OR. (IBUFCP .GT. 0))) THEN
        CALL DPGRPH(VPXDOO,VPXDOW,VPXCOO,VPXCOW,VPXOFF,VPXAXW,VPXFUL,
     .              VPYOFF,VPYAXH,VPYFUL,EMIN,EMAX,ENERGY,EFERMI,ETCMIN,
     .              ETCMAX,ETCDIS,ETCDS2,DTCMIN,DTCMAX,DTCDIS,DTCDS2,
     .              CTCMIN,CTCMAX,CTCDIS,CTCDS2,XDOSMN,XDOSMX,
     .              XCOPMX,XRANGE,DOSMN,DOSMX,COOPMX,TEXTXY,TXTANG,
     .              DOFFST,
     .              NMESH,IBUFDS,IBUFCP,ITXT,ITSDLF,ITSDMD,ITSDRT,
     .              ITSDUP,ITSDLO,ITSCUP,ITSCLO,ILTYPE,ISHADE,IBFNDX,
     .              ITXPOS,CDOSTI,CCOPTI,CENETI,CTEXT,BUFTIT,
     .              FSELEC,FARBTR,FCLOSE,FFLINE)

C
C ---------------------------------------------------------------------
C Delete buffer.
C
      ELSE IF ((ISEL .EQ. IMDELE) .AND.
     .         ((IBUFDS .GT. 0) .OR. (IBUFCP .GT. 0))) THEN
        IOPT=3
        CALL DPSELC(DOSMN,DOSMX,COOPMX,IBUFDS,IBUFCP,ILTYPE,ISHADE,
     .              IBFNDX,IOPT,NMESH,BUFTIT,FARBTR,FCLOSE)
        IF ((IBUFDS .EQ. 0) .AND. (IBUFCP .GT. 0)) THEN
          VPXCOO=VPXOFF
          VPXCOW=VPXAXW
        ITSDMD=0
        ELSE IF ((IBUFDS .GT. 0) .AND. (IBUFCP .EQ. 0)) THEN
          VPXDOO=VPXOFF
          VPXDOW=VPXAXW
          ITSDMD=0
        ELSE
        ENDIF
C
C NAG/GKS version.
C
C        CALL J06WDF
C
C PGPLOT version.
C
        CALL PGPAGE
        IF (IBUFDS+IBUFCP .GT. 0) THEN
          CALL DPRDRW(VPXDOO,VPXDOW,VPXCOO,VPXCOW,VPXOFF,VPXAXW,VPXFUL,
     .                VPYOFF,VPYAXH,VPYFUL,EMIN,EMAX,ENERGY,EFERMI,
     .                ETCMIN,ETCMAX,ETCDIS,ETCDS2,DTCMIN,DTCMAX,DTCDIS,
     .                DTCDS2,CTCMIN,CTCMAX,CTCDIS,CTCDS2,XDOSMN,XDOSMX,
     .                XCOPMX,XRANGE,DOSMN,DOSMX,COOPMX,TEXTXY,TXTANG,
     .                DOFFST,
     .                NMESH,IBUFDS,IBUFCP,ITXT,ITSDLF,ITSDMD,ITSDRT,
     .                ITSDUP,ITSDLO,ITSCUP,ITSCLO,ILTYPE,ISHADE,IBFNDX,
     .                ITXPOS,CDOSTI,CCOPTI,CENETI,CTEXT,BUFTIT,
     .                FSELEC,FARBTR,FCLOSE,FFLINE)
        ENDIF
C
C ---------------------------------------------------------------------
C Generate output file.
C
      ELSE IF (ISEL .EQ. IMOUTP) THEN
  550   WRITE(6,560)
C
C NAG/GKS version.
C
C  560   FORMAT(/,' Output file options:',/,
C     .           ' (1) Postscript',/,
C     .           ' (2) Encapsulated Postscript',/,
C     .           ' (3) Computer Graphics Metafile (CGM)',/)
C
C PGPLOT version.
C
  560   FORMAT(/,' Output file options:',/,
     .           ' (1) Gray scale postscript/encapsulated postscript',/,
     .           ' (2) Color postscript/encapsulated postscript',/,
     .           ' (3) HPGL',/)

        WRITE(6,'('' Selection: '',$)')
        READ(5,'(I1)',ERR=550) IOUSEL
        IF ((IOUSEL .LT. 1) .OR. (IOUSEL .GT. 3)) GOTO 550
C
C Temporarily store old viewport settings.
C
        VPXDS1=VPXDOO
        VPXDS2=VPXDOW
        VPXCP1=VPXCOO
        VPXCP2=VPXCOW
C
C Deactivate screen output workstation.
C Open and activate output file workstation.
C
C NAG/GKS version.
C
C        IWSID=1
C        CALL GDAWK(IWSID)
C        IWSID=2
C
C PGPLOT version.
C Comment out next IF and corresponding ENDIF statement.
C
C        IF ((IOUSEL .EQ. 1) .OR. (IOUSEL .EQ. 2)) THEN
C
  590     WRITE(6,'('' Portrait or landscape plot (p/l)? '',$)')
          READ(5,'(A1)') CANSWR
          IF (INDEX('PpLl',CANSWR) .EQ. 0) GOTO 590

          IORIEN=2
          IF (INDEX('Ll',CANSWR) .GT. 0) THEN
            IORIEN=1
            VPXFUL=VHXFUL
            VPYFUL=VHYFUL
            VPXAXW=VHXAXW
            VPYAXH=VHYAXH
            VPXOFF=VHXOFF
            VPYOFF=VHYOFF
            VPXDOW=VPXAXW
            VPXCOW=VPXAXW
            IF (VPXDOO .LE. VPXCOO) THEN
              VPXDOO=VPXOFF
              VPXCOO=VPXOFF
              IF ((IBUFCP .GT. 0) .AND. (IBUFDS .NE. 0)) THEN
                VPXDOW=SCDOS*VPXAXW
                VPXCOO=VPXDOO+VPXDOW
                VPXCOW=SCCOOP*VPXAXW
              ENDIF
            ELSE
              VPXDOO=VPXOFF
              VPXCOO=VPXOFF
              IF ((IBUFDS .GT. 0) .AND. (IBUFCP .NE. 0)) THEN
                VPXCOW=SCCOOP*VPXAXW
                VPXDOW=SCDOS*VPXAXW
                VPXDOO=VPXCOO+VPXCOW
              ENDIF
            ENDIF
C
C NAG/GKS version.
C comment out next IF/ENDIF block.
C
C            IF (IOUSEL .EQ. 1) THEN
C              CALL GOPWK(IWSID,IPSCON,IPSA4L)
C            ELSE
C              CALL GOPWK(IWSID,IEPSCON,IEPSA4L)
C            ENDIF
          ELSE
C
C NAG/GKS version.
C comment out next IF/ENDIF block.
C
C            IF (IOUSEL .EQ. 1) THEN
C              CALL GOPWK(IWSID,IPSCON,IPSA4P)
C            ELSE
C              CALL GOPWK(IWSID,IEPSCON,IEPSA4P)
C            ENDIF
           ENDIF
C
C NAG/GKS version.
C Comment out next ELSE IF/ENDIF block
C        ELSE IF (IOUSEL .EQ. 3) THEN
C
C Metafile.
C
C          CALL GOPWK(IWSID,ICGCON,ICGM)
C        ENDIF
C
C NAG/GKS version.
C
C        CALL GACWK(IWSID)
C
C Ask for area shade scale factor.
C
C NAG/GKS version.
C
C  620   WRITE(6,630)
C  630   FORMAT(' Area shade scale factor (0.0-1.0)? ',$)
C        READ(6,640) ARSCAL
C  640   FORMAT(F10.5)
C        IF ((ARSCAL .LE. 0.0) .OR. (ARSCAL .GT. 1.0)) GOTO 620
C
C        CALL J06XMF(ARSCAL)
C end NAG/GKS version.
C
C PGPLOT version.
C
        IDEVPR=PGOPEN(SDEVIC(IORIEN,IOUSEL))
        CALL PGSLCT(IDEVPR)
        CALL PGPAP(PAPERW,ASPECT)
        CALL DPRDRW(VPXDOO,VPXDOW,VPXCOO,VPXCOW,VPXOFF,VPXAXW,VPXFUL,
     .              VPYOFF,VPYAXH,VPYFUL,EMIN,EMAX,ENERGY,EFERMI,ETCMIN,
     .              ETCMAX,ETCDIS,ETCDS2,DTCMIN,DTCMAX,DTCDIS,DTCDS2,
     .              CTCMIN,CTCMAX,CTCDIS,CTCDS2,XDOSMN,XDOSMX,
     .              XCOPMX,XRANGE,DOSMN,DOSMX,COOPMX,TEXTXY,TXTANG,
     .              DOFFST,
     .              NMESH,IBUFDS,IBUFCP,ITXT,ITSDLF,ITSDMD,ITSDRT,
     .              ITSDUP,ITSDLO,ITSCUP,ITSCLO,ILTYPE,ISHADE,IBFNDX,
     .              ITXPOS,CDOSTI,CCOPTI,CENETI,CTEXT,BUFTIT,
     .              FSELEC,FARBTR,FCLOSE,FFLINE)
C
C Deactivate and close output file workstation.
C Activate screen output workstation and redraw graph.
C
C NAG/GKS version.
C 
C        CALL GDAWK(IWSID)
C        CALL GCLWK(IWSID)
C        IWSID=1
C        CALL GACWK(IWSID)
C        CALL J06WDF
C
C PGPLOT version.
C
        CALL PGCLOS
        CALL PGSLCT(IDEVSC)

        VPXDOO=VPXDS1
        VPXDOW=VPXDS2
        VPXCOO=VPXCP1
        VPXCOW=VPXCP2
        VPXFUL=VVXFUL
        VPYFUL=VVYFUL
        VPXAXW=VVXAXW
        VPYAXH=VVYAXH
        VPXOFF=VVXOFF
        VPYOFF=VVYOFF
C
C Set old pattern size
C
C NAG/GKS version.
C
C        CALL J06XMF(1.0)
C
        CALL DPRDRW(VPXDOO,VPXDOW,VPXCOO,VPXCOW,VPXOFF,VPXAXW,VPXFUL,
     .            VPYOFF,VPYAXH,VPYFUL,EMIN,EMAX,ENERGY,EFERMI,ETCMIN,
     .            ETCMAX,ETCDIS,ETCDS2,DTCMIN,DTCMAX,DTCDIS,DTCDS2,
     .            CTCMIN,CTCMAX,CTCDIS,CTCDS2,XDOSMN,XDOSMX,
     .            XCOPMX,XRANGE,DOSMN,DOSMX,COOPMX,TEXTXY,TXTANG,
     .            DOFFST,
     .            NMESH,IBUFDS,IBUFCP,ITXT,ITSDLF,ITSDMD,ITSDRT,
     .            ITSDUP,ITSDLO,ITSCUP,ITSCLO,ILTYPE,ISHADE,IBFNDX,
     .            ITXPOS,CDOSTI,CCOPTI,CENETI,CTEXT,BUFTIT,
     .            FSELEC,FARBTR,FCLOSE,FFLINE)
C
C ---------------------------------------------------------------------
C Save graph layout.
C
      ELSE IF (ISEL .EQ. IMSAVE) THEN
C
C VMS and AIX version: Remove FILE='fort.7'
C MS-Windows version: Add FILE='fort.7' to the OPEN statement.
C
        OPEN(UNIT=7,FORM='formatted',STATUS='unknown',FILE='fort.7')
C        OPEN(UNIT=7,FORM='formatted',STATUS='unknown')
        REWIND 7
        WRITE(7,650) IBUFDS
  650   FORMAT(I2)
        WRITE(7,660) (IBFNDX(I),ILTYPE(I),ISHADE(I),FARBTR(I),FCLOSE(I),
     .                DOFFST(I),BUFMIN(I),BUFMAX(I),BUFTIT(I),
     .                I=1,IBUFDS)
  660   FORMAT(3I2,2L1,3F12.5,A72)
        WRITE(7,650) IBUFCP
        WRITE(7,670) (IBFNDX(I),ILTYPE(I),ISHADE(I),FARBTR(I),FCLOSE(I),
     .                BUFMIN(I),BUFMAX(I),BUFTIT(I),
     .                I=MAXBUF+1,MAXBUF+IBUFCP)
  670   FORMAT(3I2,2L1,2F12.5,A72)
        WRITE(7,680) (FSELEC(I),I=1,MAXSEL)
  680   FORMAT(20L1)
        WRITE(7,690) VPXDOO,VPXDOW,VPXCOO,VPXCOW,VPXOFF,VPXAXW,VPXFUL,
     .               VPYOFF,VPYAXH,VPYFUL
  690   FORMAT(10F10.5)
        WRITE(7,700) EMIN,EMAX
  700   FORMAT(2F10.5)
        WRITE(7,710) ETCMIN,ETCMAX,ETCDIS,ETCDS2
        WRITE(7,710) DTCMIN,DTCMAX,DTCDIS,DTCDS2
        WRITE(7,710) CTCMIN,CTCMAX,CTCDIS,CTCDS2
  710   FORMAT(4F10.5)
        WRITE(7,720) XDOSMN,XDOSMX,XCOPMX,XRANGE,DOSMN,DOSMX,COOPMX
  720   FORMAT(7F10.5)
        WRITE(7,730) EFERMI,FFLINE
  730   FORMAT(F10.5,L1)
        WRITE(7,740) ITSDLF,ITSDMD,ITSDRT,ITSDUP,ITSDLO,ITSCUP,ITSCLO
  740   FORMAT(7I1)
        WRITE(7,750) CDOSTI
        WRITE(7,750) CCOPTI
        WRITE(7,750) CENETI
  750   FORMAT(A80)
        WRITE(7,760) ITXT
  760   FORMAT(I4)
        WRITE(7,770) (TEXTXY(I,1),TEXTXY(I,2),TXTANG(I),ITXPOS(I),
     .                CTEXT(I),I=1,ITXT)
  770   FORMAT(3F10.5,I2,A80)
        CLOSE(7)
C
C ---------------------------------------------------------------------
C Load graph layout.
C
      ELSE IF (ISEL .EQ. IMLOAD) THEN
C
C VMS and AIX version: Remove FILE='fort.7'
C MS-Windows version: Add FILE='fort.7' to the OPEN statement.
C
        OPEN(UNIT=7,FORM='formatted',STATUS='unknown',FILE='fort.7',
     .       ERR=860)
C        OPEN(UNIT=7,FORM='formatted',STATUS='unknown',ERR=860)
        READ(7,650) IBUFDS
        READ(7,660) (IBFNDX(I),ILTYPE(I),ISHADE(I),FARBTR(I),FCLOSE(I),
     .               DOFFST(I),BUFMIN(I),BUFMAX(I),BUFTIT(I),
     .               I=1,IBUFDS)
        READ(7,650) IBUFCP
        READ(7,670) (IBFNDX(I),ILTYPE(I),ISHADE(I),FARBTR(I),FCLOSE(I),
     .               BUFMIN(I),BUFMAX(I),BUFTIT(I),
     .               I=MAXBUF+1,MAXBUF+IBUFCP)
        READ(7,680) (FSELEC(I),I=1,MAXSEL)
        READ(7,690) VPXDOO,VPXDOW,VPXCOO,VPXCOW,VPXOFF,VPXAXW,VPXFUL,
     .              VPYOFF,VPYAXH,VPYFUL
        READ(7,700) EMIN,EMAX
        READ(7,710) ETCMIN,ETCMAX,ETCDIS,ETCDS2
        READ(7,710) DTCMIN,DTCMAX,DTCDIS,DTCDS2
        READ(7,710) CTCMIN,CTCMAX,CTCDIS,CTCDS2
        READ(7,720) XDOSMN,XDOSMX,XCOPMX,XRANGE,DOSMN,DOSMX,COOPMX
        READ(7,730) EFERMI,FFLINE
        READ(7,740) ITSDLF,ITSDMD,ITSDRT,ITSDUP,ITSDLO,ITSCUP,ITSCLO
        READ(7,750) CDOSTI
        READ(7,750) CCOPTI
        READ(7,750) CENETI
        READ(7,760) ITXT
        READ(7,770) (TEXTXY(I,1),TEXTXY(I,2),TXTANG(I),ITXPOS(I),
     .              CTEXT(I),I=1,ITXT)
        CLOSE(7)
        IF (IBUFDS .GT. 0) THEN
          DO I=1,IBUFDS
            IFINDX=IBFNDX(I)
            REWIND 1
            DO J=1,3*MXTITL*MXCARD+IFINDX*NMESH+4
              READ(1,'(A80)') CDUMMY
            END DO
            DO J=1,NMESH
              READ(1,'(F12.5)') BUFFER((I-1)*NMESH+J)
            END DO
          END DO
        ENDIF
        IF (IBUFCP .GT. 0) THEN
          DO I=1,IBUFCP
            IFINDX=IBFNDX(MAXBUF+I)
            REWIND 1
            DO J=1,3*MXTITL*MXCARD+IFINDX*NMESH+4
              READ(1,'(A80)') CDUMMY
            END DO
            DO J=1,NMESH
              READ(1,'(F12.5)') BUFFER((MAXBUF+I-1)*NMESH+J)
            END DO
          END DO
        ENDIF
        CALL DPRDRW(VPXDOO,VPXDOW,VPXCOO,VPXCOW,VPXOFF,VPXAXW,VPXFUL,
     .              VPYOFF,VPYAXH,VPYFUL,EMIN,EMAX,ENERGY,EFERMI,ETCMIN,
     .              ETCMAX,ETCDIS,ETCDS2,DTCMIN,DTCMAX,DTCDIS,DTCDS2,
     .              CTCMIN,CTCMAX,CTCDIS,CTCDS2,XDOSMN,XDOSMX,
     .              XCOPMX,XRANGE,DOSMN,DOSMX,COOPMX,TEXTXY,TXTANG,
     .              DOFFST,
     .              NMESH,IBUFDS,IBUFCP,ITXT,ITSDLF,ITSDMD,ITSDRT,
     .              ITSDUP,ITSDLO,ITSCUP,ITSCLO,ILTYPE,ISHADE,IBFNDX,
     .              ITXPOS,CDOSTI,CCOPTI,CENETI,CTEXT,BUFTIT,
     .              FSELEC,FARBTR,FCLOSE,FFLINE)
        GOTO 880
  860   WRITE(6,'('' Layout file does not exist. First use option ('',
     .        I2,'').'')') IMSAVE
  880   CONTINUE
      ENDIF


      IF (ISEL .NE. 0) GOTO 70
C
C Terminate plotting and close graphic system.
C
C NAG/GKS version.
C
C      CALL J06WZF
C
C PGPLOT version.
C
      CALL PGEND
      STOP
  890 WRITE(6,'('' Inputfile could not be opened.'')')
      STOP
      END
