C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  GETVAR                                                          *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Reads geometry variation parameters.               *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs)                              *
C *  Available..: Calzaferri group, Institute for Inorganic and Phy- *
C *               sical Chemistry, Freiestr. 3, CH-3012 Berne,       *
C *               Switzerland                                        *
C *  Date.......: 06.07.1994                                         *
C *  Revision...: 09.08.1996 (Ruedi Rytz)                            *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: PART1                                              *
C *                                                                  *
C *  Calls......: READL,WRITEL                                       *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE GETVAR
 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
 
      INCLUDE 'sizes'
 
      CHARACTER*80 ELINE
 
      COMMON /IOUNIT/IIN,IOUT,ISTART(40)
 
      COMMON /KEYWRD/KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80
 
      COMMON /VARGEO/VARINC(NVARMX),IVRLIN(NVARMX),ISTPMX(NVARMX),
     .               IVARTP(NVARMX),ISTEPS(NVARMX),IVARY,NVARY,
     .               NVARLN,NNCPL,LCOUPL(NVARMX+1)
      LOGICAL LCOUPL
C
C Explanation of variables:
C VARINC: Increment of coordinate to be varied
C IVRLIN: Line to be varied
C ISTPMX: Maximum number of variation steps for corresponding line.
C IVARTP: Component to be varied:
C         ZMAT:  1=LENGTH, 2=ANGLE, 3=DIHEDRAL
C         XYZ:   1=X, 2=Y, 3=Z
C         FRACT: 1=X, 2=Y, 3=Z, 4=A, 5=B, 6=C, 7=ALPHA, 8=BETA, 9=GAMMA
C ISTEPS: Actual variation step for corresponding line.
C IVARY:  Actual number of performed variation steps.
C NVARY:  Total number of variation steps.
C NVARLN: Number of lines to be varied.
C NNCPL:  Number of uncoupled lines to be varied.
C LCOUPL: .TRUE.=Line coupled to preceding line.
C
      COMMON /ZMAT/BL(NATMMX),ALPH(NATMMX),BET(NATMMX),IZ(NATMMX,3),NZ
C
C Initialize data.
C
      NVARLN=0
C      NVARY=1
      NVARY=0
      NNCPL=0

      DO 10 I=1,NVARMX+1
        LCOUPL(NVARMX)=.FALSE.
   10 CONTINUE
 
   20 IF (INDEX(SLINE,'VARY') .GT. 0) THEN
        IF (NVARLN .EQ. 0) WRITE(IOUT,30)
   30   FORMAT(/,2X,'Geometry variation parameters:',/)
        NVARLN=NVARLN+1
        IP=ISTART(2)
        IVRLIN(NVARLN)=INT(READA(SLINE,IP,80))
        IF (IVRLIN(NVARLN) .GT. NZ) THEN
          ELINE='Number of line to be varied larger than maximum '//
     .          'number of POS cards.'
          CALL WRITEL(ELINE)
        ENDIF
        IP1=ISTART(3)
        IP2=ISTART(4)-2
        IF (INDEX(KEYWRD,'ZMAT') .GT. 0) THEN
          IF (SLINE(IP1:IP2) .EQ. 'LENGTH') THEN
            IVARTP(NVARLN)=1
          ELSE IF (SLINE(IP1:IP2) .EQ. 'ANGLE') THEN
            IVARTP(NVARLN)=2
          ELSE IF (SLINE(IP1:IP2) .EQ. 'DIHEDRAL') THEN
            IVARTP(NVARLN)=3
          ELSE
            ELINE=SLINE(IP:IP2)//': Wrong component name.'
            CALL WRITEL(ELINE)
          ENDIF
        ELSE IF (INDEX(KEYWRD,'XYZ') .GT. 0) THEN
          IF (SLINE(IP1:IP2) .EQ. 'X') THEN
            IVARTP(NVARLN)=1
          ELSE IF (SLINE(IP1:IP2) .EQ. 'Y') THEN
            IVARTP(NVARLN)=2
          ELSE IF (SLINE(IP1:IP2) .EQ. 'Z') THEN
            IVARTP(NVARLN)=3
          ELSE
            ELINE=SLINE(IP1:IP2)//': Wrong component name.'
            CALL WRITEL(ELINE)
          ENDIF
        ELSE
          IF (SLINE(IP1:IP2) .EQ. 'X') THEN
            IVARTP(NVARLN)=1
          ELSE IF (SLINE(IP1:IP2) .EQ. 'Y') THEN
            IVARTP(NVARLN)=2
          ELSE IF (SLINE(IP1:IP2) .EQ. 'Z') THEN
            IVARTP(NVARLN)=3
          ELSE IF (SLINE(IP1:IP2) .EQ. 'A') THEN
            IVRLIN(NVARLN)=-1
            IVARTP(NVARLN)=4
          ELSE IF (SLINE(IP1:IP2) .EQ. 'B') THEN
            IVRLIN(NVARLN)=-1
            IVARTP(NVARLN)=5
          ELSE IF (SLINE(IP1:IP2) .EQ. 'C') THEN
            IVRLIN(NVARLN)=-1
            IVARTP(NVARLN)=6
          ELSE IF (SLINE(IP1:IP2) .EQ. 'ALPHA') THEN
            IVRLIN(NVARLN)=-1
            IVARTP(NVARLN)=7
          ELSE IF (SLINE(IP1:IP2) .EQ. 'BETA') THEN
            IVRLIN(NVARLN)=-1
            IVARTP(NVARLN)=8
          ELSE IF (SLINE(IP1:IP2) .EQ. 'GAMMA') THEN
            IVRLIN(NVARLN)=-1
            IVARTP(NVARLN)=9
          ELSE
            ELINE=SLINE(IP1:IP2)//': Wrong component name.'
            CALL WRITEL(ELINE)
          ENDIF          
        ENDIF
        IP=ISTART(4)
        VARINC(NVARLN)=READA(SLINE,IP,80)
        IP=ISTART(5)
        ISTPMX(NVARLN)=INT(READA(SLINE,IP,80))
        IF (ISTPMX(NVARLN) .LT. 1) THEN
          ELINE='Wrong number of variation steps.'
          CALL WRITEL(ELINE)
        ENDIF
        ISTEPS(NVARLN)=1
C
C        IF (NVARLN .EQ. 1) THEN
C
        IF (NVARY .EQ. 0) THEN
          NVARY=ISTPMX(NVARLN)
          NNCPL=NNCPL+1
        ELSE

C          IP3=ISTART(6)
C          IP4=ISTART(7)-2
C          IF (SLINE(IP3:IP4) .EQ. 'COUPLED') THEN

          IF (INDEX(SLINE,'COUPLED') .GT. 0) THEN
            LCOUPL(NVARLN)=.TRUE.
          ELSE
            NVARY=NVARY*ISTPMX(NVARLN)
            NNCPL=NNCPL+1
          ENDIF
        ENDIF
 
        IF (LCOUPL(NVARLN)) THEN
          WRITE(IOUT,40) IVRLIN(NVARLN),SLINE(IP1:IP2),VARINC(NVARLN),
     .                   ISTPMX(NVARLN)
   40     FORMAT(2X,'Line ',I3,': ',A8,' by ',F10.5,' in ',I3,' steps ',
     .           'coupled to preceding line.')
        ELSE
          WRITE(IOUT,50) IVRLIN(NVARLN),SLINE(IP1:IP2),VARINC(NVARLN),
     .                   ISTPMX(NVARLN)
   50     FORMAT(2X,'Line ',I3,': ',A8,' by ',F10.5,' in ',I3,' steps.')
        ENDIF
 
        CALL READL
        GOTO 20
      ENDIF

      RETURN
      END
