C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  OMATBP                                                          *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Output of CO coefficients.                         *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987,06.05.92/mbs (Martin Braendle, c/o       *
C *               Calzaferri group, Inst. f. Inorg. and Phys. Chem., *
C *               Freiestr. 3, CH-3012 Berne)                        *
C *  Remarks....: Slight modification of COMMON blocks.              *
C *                                                                  *
C *  Called from: CRYST                                              *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************

      SUBROUTINE OMATBP(Z,X,Y,NN,LBR,LER,LB,LE,IMAT,MM)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'

      DIMENSION Z(NN),X(NN,NN),Y(NN,NN),IMAT(NDIMMX),PHASE(3),RAP(3)
      CHARACTER TEXT*10,LEG*12

      COMMON /IOUNIT/IIN,IOUT,ISTART(40)
      COMMON /DELET/IND(NDIMMX),INDIM,IVAL,NORBE(NATMMX),NDEL
      COMMON /ATOM1/NS(NSYMMX,NAEQMX),NP(NSYMMX),ND(NSYMMX),NF(NSYMMX),
     .              INATOM,ISYM(NATMMX),NELEM
      COMMON /AOOUT1/INDTS(NDIMMX)

      COMMON /ATO/AC(NATMMX),SYMBOL(NSYMMX)
      CHARACTER AC*7,SYMBOL*2

      COMMON /AOOUT2/INDT(NDIMMX,2)
      CHARACTER INDT*10

      PI=3.1415926535898D0
      LEG='real    imag    norm    phase'
      IFLAG=1
      ILOWER=LB
      NL=LE

   10 IUPPER=ILOWER+2
      IF (IUPPER-NL) 30,20,20
   20 IUPPER=NL
      IFLAG=0
   30 WRITE(IOUT,40) (J,J=ILOWER-MM+1,IUPPER-MM+1)
   40 FORMAT(34X,2(I10,24X),I10)
      WRITE(IOUT,50) (Z(J),J=ILOWER-MM+1,IUPPER-MM+1)
   50 FORMAT(34X,2(F14.8,20X),F14.8)
      WRITE(IOUT,60) (LEG,J=ILOWER-MM+1,IUPPER-MM+1)
   60 FORMAT(26X,2(3X,A30,1X),3X,A30)

      IL1=ILOWER
      IU1=IUPPER

      TEXT=' '
      DO 110 I=LBR,LER
        IF (IMAT(I) .EQ. 0) GOTO 110
        II=I-MM+1

        DO 70 J=IL1,IU1
          JJ=J-IL1+1
          ISGX=1
          IF (X(I,J) .LT. 0.0D0) ISGX=-1
          ISGY=1
          IF (Y(I,J) .LT. 0.0D0) ISGY=-1
          RAP(JJ)=DSQRT(X(I,J)*X(I,J)+Y(I,J)*Y(I,J))
          IF (DABS(X(I,J)) .LT. 1.0D-38) THEN
            IF (ISGY .EQ. 1) PHASE(JJ)=90.0D0
            IF (ISGY .EQ. -1) PHASE(JJ)=-90.0D0
            IF (DABS(Y(I,J)) .LT. 1.0D-38) PHASE(JJ)=0.0D0
          ELSE
            TA=Y(I,J)/X(I,J)
            ALP=DATAN(TA)*180.0D0/PI
            IF ((ISGX .EQ. -1) .AND. (ISGY .EQ. -1)) ALP=ALP+180.0D0
            IF ((ISGX .EQ. 1) .AND. (ISGY .EQ. -1)) ALP=ALP+360.0D0
            IF ((ISGX .EQ. -1) .AND. (ISGY .EQ. 1)) ALP=ALP+180.0D0
            PHASE(JJ)=ALP
            IF (PHASE(JJ) .GT. 359.999D0) PHASE(JJ)=PHASE(JJ)-360.0D0
          ENDIF
   70   CONTINUE

        IF (INDT(I,1) .EQ. TEXT) THEN
          WRITE(IOUT,80) INDT(I,2),II,(X(I,J),Y(I,J),RAP(J-IL1+1),
     .                   PHASE(J-IL1+1),J=IL1,IU1)
   80     FORMAT(11X,A10,1X,I3,1X,2(3F8.4,F8.2,2X),3F8.4,F8.2)
        ELSE
          WRITE(IOUT,90)
   90     FORMAT(' ')
          WRITE(IOUT,100) (INDT(I,L),L=1,2),II,(X(I,J),Y(I,J),
     .                    RAP(J-IL1+1),PHASE(J-IL1+1),J=IL1,IU1)
  100     FORMAT(1X,2A10,1X,I3,1X,2(3F8.4,F8.2,2X),3F8.4,F8.2)
        ENDIF
        TEXT=INDT(I,1)
  110 CONTINUE
      WRITE(IOUT,90)

      IF (IFLAG) 120,130,120
  120 ILOWER=ILOWER+3
      GOTO 10

  130 RETURN
      END
