C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  POPULA                                                          *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Does population analysis for MO calculations.      *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 06.05.92/mbs (Martin Braendle, c/o      *
C *               Calzaferri group, Inst. f. Inorg. and Phys. Chem., *
C *               Freiestr. 3, CH-3012 Berne)                        *
C *  Remarks....: Slight modification of COMMON blocks.              *
C *                                                                  *
C *  Called from: HUCKEL                                             *
C *                                                                  *
C *  Calls......: MOPOP                                              *
C *                                                                  *
C ********************************************************************

      SUBROUTINE POPULA(D,F,V,DD,NDIM,NATOM,NELEC,IST)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'

      DIMENSION D(NDIM,NDIM),F(NDIM,NDIM),V(NDIM,NDIM),DD(NDIM)
      INTEGER SORB(NATMMX)

      COMMON /IOUNIT/IIN,IOUT,ISTART(40)
      COMMON /ATOM1/NS(NSYMMX,NAEQMX),NP(NSYMMX),ND(NSYMMX),NF(NSYMMX),
     .              INATOM,ISYM(NATMMX),NELEM

      COMMON /ATO/AC(NATMMX),SYMBOL(NSYMMX)
      CHARACTER AC*7,SYMBOL*2

      NOC=(NELEC-IST)/2

      READ(21) (SORB(I),I=1,NATOM),V 
      REWIND 21

      DO 30 I=1,NDIM
        DO 30 J=1,I
          D(I,J)=0.0D0
          DO 10 K=1,NOC
   10     D(I,J)=D(I,J)+F(I,K)*F(J,K)*2.0D0
          IF (IST .NE. 0) THEN
            DO 20 IJ=1,IST
              D(I,J)=D(I,J)+F(I,NOC+IJ)*F(J,NOC+IJ)
   20       CONTINUE
          ENDIF
          D(J,I)=D(I,J)
   30 CONTINUE

      DO 40 I=1,NDIM
        DO 40 J=1,I
          F(I,J)=V(I,J)*D(I,J)
   40 F(J,I)=F(I,J)

      DO 60 I=1,NATOM
        NFI=SORB(I)
        NLI=SORB(I+1)-1
        IF (I .EQ. NATOM) NLI=NDIM
        DO 60 J=1,I
          NFJ=SORB(J)
          NLJ=SORB(J+1)-1
          IF (J .EQ. NATOM) NLJ=NDIM
          D(I,J)=0.0D0
          DO 50 K=NFI,NLI
            DO 50 L=NFJ,NLJ
   50     D(I,J)=D(I,J)+F(K,L)
   60 D(J,I)=D(I,J)

      DO 80 I=1,NATOM
        DO 70 J=1,NATOM
          F(I,J)=D(I,J)*2.0D0
   70   CONTINUE
   80 CONTINUE

      WRITE(IOUT,90)
   90 FORMAT(//,5X,'Population between atoms',/)
      CALL MOPOP(F,NDIM,NATOM,NATOM)

      DO 110 I=1,NATOM
        DD(I)=0.0D0
        DO 100 J=1,NATOM
  100   DD(I)=DD(I)+D(I,J)
  110 CONTINUE

      WRITE(IOUT,120)
  120 FORMAT(//,5X,'Gross population of atoms',/)
      DO 130 I=1,NATOM
  130 WRITE(IOUT,140) AC(I),DD(I),I
  140 FORMAT(10X,A7,F12.6,1X,'(',I3,')')

      RETURN
      END
