C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  FUNCTION REPUL2                                                 *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Calculates part of integrals for core-core         *
C *               repulsion:  exp(-2*zeta*R)/(2*n*R) *               *
C *               * sum over [(2*R*zeta)**(2*n-p)*p/(2*n-p)!]        *
C *                                                                  *
C *  Authors....: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: -                                                  *
C *  Revision...: 11.06.92/mbs                                       *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: REPUL1                                             *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************

      DOUBLE PRECISION FUNCTION REPUL2(N,R,SE)
      
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      DIMENSION FAC(0:13)

      DATA FAC/1.0D0,1.0D0,2.0D0,6.0D0,24.0D0,120.0D0,720.0D0,5040.0D0,
     .         40320.D0,362880.0D0,3628800.0D0,39916800.0D0,
     .         479001600.0D0,6227020800.0D0/

C
C N:  main quantum number
C R:  distance between two atoms
C SE: Slaterexponent*2, In case of the cross term of double-zeta orbitals 
C     zeta1+zeta2
C

      N2=2*N
      SUM=0.0D0

      DO 10 IP=1,N2
        J=N2-IP
        RES=DBLE(IP)/FAC(J)
        RES1=(R*SE)**DBLE(J)
        SUM=SUM+RES*RES1
   10 CONTINUE
      RES2=DEXP(-SE*R)/(DBLE(N2)*R)
      REPUL2=RES2*SUM
      RETURN
      END
