C ********************************************************************
C * EHPC - Extended Hueckel properties calculations                  *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  PPART                                                           *
C *                                                                  *
C *  Routine of EHPC                                                 *
C *                                                                  *
C *  Usage......: Controls the flow of band property analysis, also  *
C *               orders the crystal orbital energies calculated for *
C *               a set of different k points                        *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 30.03.92/mbs (Martin Braendle c/o       *
C *               Calzaferri group, Inst. Inorg. and Phys. Chem.,    *
C *               Freiestr. 3, CH-3012 Berne), 12.06.92/mbs,         *
C *               14.10.93/mbs                                       *
C *  Remarks....: I/O will be formatted. Slight modification of      *
C *               COMMON blocks.                                     *
C *                                                                  *
C *  Called from: PARRAY                                             *
C *                                                                  *
C *  Calls......: GPROJ,GPROJF,GPROJB,INTEGR,PROJT,PROJTB,SMOOTH,    *
C *               SMOOTO,SMOOTB                                      *
C *                                                                  *
C ********************************************************************

      SUBROUTINE PPART(ENDK,ID,W,D,IT,A,IACC,A1,A2,A3,A4,A5,A6,B1,B2,
     .                 B3,B4,NDIM,NKP,NDK,NSTEP)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'

      DIMENSION ENDK(NDK),ID(NDK),W(NDK),D(NDIM),IT(NDK),A(NDIM,NDIM),
     .          IACC(NKP,NSTEP),A1(NSTEP),A2(NSTEP),A3(NSTEP),A4(NSTEP),
     .          A5(NSTEP),A6(NSTEP),B1(NMESH),B2(NMESH),B3(NMESH),
     .          B4(NMESH),XK(3)
      CHARACTER CDUMMY*80

      COMMON /AOPRF/ IPROJF(NDIMMX,NFMOMX),NDPRF(NFMOMX),NPROJF
      COMMON /AOPROJ/ IPROJ(NDIMMX,NFMOMX),NDPR(NFMOMX),NPROJ

      COMMON /ATO/ AC(NATMMX),SYMBOL(NSYMMX)
      CHARACTER AC*7,SYMBOL*2

      COMMON /ATOM1/ NSS(NSYMMX),NPP(NSYMMX),NDD(NSYMMX),NFF(NSYMMX),
     .               IATOM,ISYM(NATMMX),NELEM
      COMMON /IOUNIT/ IIN,IOUT,ISTART(40)

      COMMON /KEYWRD/ KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80

      COMMON /OPTV/ DELTA,EMIN,EMAX,STEP,XLIMIT,LEVB,LEVE,NMESH,IVARYS,
     .              IVARYE,NVARY

      COMMON /ORB1/ INDT(NDIMMX,2)
      CHARACTER INDT*10

      COMMON /ORB2/ NATOM,IND(NDIMMX),NORBEF(NATMMX),NORBE(NATMMX),NDEL
      COMMON /OVLP/ IPROJB(NBONMX,MXCARD),NDPRB(MXCARD),NPROJB,
     .              NBTOT(MXCARD)
      COMMON /TIME/ XTIME
      COMMON /TIT1/ NCARD(MXCARD),NCARDF(MXCARD),NCARDB(MXCARD)

      COMMON /TIT2/ TITL(MXCARD,MXTITL),TITLF(MXCARD,MXTITL),
     .              TITLB(MXCARD,MXTITL)
      CHARACTER TITL*80,TITLF*80,TITLB*80

      I1=INDEX(KEYWRD,'IDOS')
      I2=INDEX(KEYWRD,'IAOPOP')
      I3=INDEX(KEYWRD,'IFMOPOP')
      I4=INDEX(KEYWRD,'IOVLPOP')
      J1=INDEX(KEYWRD,'SDOS')
      J2=INDEX(KEYWRD,'SAOPOP')
      J3=INDEX(KEYWRD,'SFMOPOP')
      J4=INDEX(KEYWRD,'SOVLPOP')

      DO 20 J=1,MXTITL
        DO 10 I=1,MXCARD
          TITL(I,J)=' '
          TITLF(I,J)=' '
          TITLB(I,J)=' '
   10   CONTINUE
   20 CONTINUE

      IF ((I2 .NE. 0) .OR. (J2 .NE. 0)) CALL GPROJ(NDIM)
      IF ((I3 .NE. 0) .OR. (J3 .NE. 0)) CALL GPROJF(NDIM)
      IF ((I4 .NE. 0) .OR. (J4 .NE. 0)) CALL GPROJB

      IF (INDEX(KEYWRD,'CHECKFILE') .NE. 0) RETURN

      IF (IVARYS .GT. 1) THEN
        DO 100 IVARY=1,IVARYS-1
          DO 60 I=1,NKP
            READ(1,30) (XK(J),J=1,3),WEI
   30       FORMAT(4F12.6)
            DO 50 J=1,NDIM
              READ(1,40) D(J)
   40         FORMAT(F12.6)
              LOC=J+(I-1)*NDIM
              ENDK(LOC)=D(J)
              W(LOC)=WEI
              ID(LOC)=LOC
   50       CONTINUE
   60     CONTINUE
          READ(1,70) IORD
   70     FORMAT(I2)
          DO 90 I=1,NDK
            READ(1,80) ID(I)
   80       FORMAT(I6)
   90     CONTINUE
  100   CONTINUE
      ENDIF

      DO 340 IVARY=IVARYS,IVARYE
        DO 120 I=1,NKP
          READ(1,30) (XK(J),J=1,3),WEI
          DO 110 J=1,NDIM
            READ(1,40) D(J)
            LOC=J+(I-1)*NDIM
            ENDK(LOC)=D(J)
            W(LOC)=WEI
            ID(LOC)=LOC
  110     CONTINUE
  120   CONTINUE

        READ(1,70) IORD
        IF (IORD .EQ. 0) THEN
          DO 200 IROW=1,NDIM
            DO 190 ICOL=1,NKP
              LOC1=(ICOL-1)*NDIM+IROW
              ID1=ID(LOC1)
              ICOLP1=ICOL+1
              ICOLM1=ICOL-1
              DO 150 JA=ICOLP1,NKP
                DO 140 LA=IROW,NDIM
                  LOC2=(JA-1)*NDIM+LA
                  ID2=ID(LOC2)
                  IF (ENDK(ID2) .GT. ENDK(ID1)) GOTO 150
                  ID(LOC2)=ID1
                  ID(LOC1)=ID2
                  ID1=ID2
                  DO 130 K=LA,NDIM-1
                    LC1=(JA-1)*NDIM+K
                    LC2=LC1+1
                    IDD1=ID(LC1)
                    IDD2=ID(LC2)
                    IF (ENDK(IDD2) .LT. ENDK(IDD1)) THEN
                      ID(LC2)=IDD1
                      ID(LC1)=IDD2
                    ENDIF
  130             CONTINUE
  140           CONTINUE
  150         CONTINUE

              IF (IROW .LT. NDIM) THEN
                DO 180 JB=1,ICOLM1
                  DO 170 LB=IROW+1,NDIM
                    LOC2=(JB-1)*NDIM+LB
                    ID2=ID(LOC2)
                    IF (ENDK(ID2) .GT. ENDK(ID1)) GOTO 180
                    ID(LOC2)=ID1
                    ID(LOC1)=ID2
                    ID1=ID2
                    DO 160 K=LB,NDIM-1
                      LC1=(JB-1)*NDIM+K
                      LC2=LC1+1
                      IDD1=ID(LC1)
                      IDD2=ID(LC2)
                      IF (ENDK(IDD2) .LT. ENDK(IDD1)) THEN
                        ID(LC2)=IDD1
                        ID(LC1)=IDD2
                      ENDIF
  160               CONTINUE
  170             CONTINUE
  180           CONTINUE
              ENDIF

  190       CONTINUE
  200     CONTINUE

          REWIND 1
          DO 220 I=1,NDIM+NELEM+IATOM+NKP*(NDIM+1)+1
            READ(1,210) CDUMMY
  210       FORMAT(A80)
  220     CONTINUE

          IORD=1
          WRITE (1,230) IORD
  230     FORMAT(I2)

          NUM=0
          DO 250 I=1,NDIM
            DO 240 J=1,NKP
              LOC=(J-1)*NDIM+I
              NUM=NUM+1
              IT(NUM)=ID(LOC)
  240       CONTINUE
  250     CONTINUE

          DO 270 I=1,NDK
            ID(I)=IT(I)
            WRITE(1,260) ID(I)
  260       FORMAT(I6)
  270     CONTINUE
        ELSE
          DO 290 I=1,NDK
            READ(1,280) ID(I)
  280       FORMAT(I6)
  290     CONTINUE
        ENDIF

        IF (IVARYS .EQ. IVARYE) WRITE(IOUT,300) IVARY
  300   FORMAT(//,' Geometry variation #',I4,//)
C
        II=I1+I2+I3+I4
        IF (II .NE. 0) CALL INTEGR (ENDK,ID,W,IACC,A1,A2,A3,A4,A5,A6,
     .  NDIM,NKP,NDK,NSTEP,IVARY)
        IF (I2 .NE. 0) CALL PROJT (W,IACC,A,A1,A2,A3,A5,A6,NDIM,NKP,
     .  NDK,NSTEP,IPROJ,NDPR,NPROJ,IVARY,2)
        IF (I3 .NE. 0) CALL PROJT (W,IACC,A,A1,A2,A3,A5,A6,NDIM,NKP,
     .  NDK,NSTEP,IPROJF,NDPRF,NPROJF,IVARY,4)
        IF (I4 .NE. 0) CALL PROJTB (W,IACC,A1,A2,A3,A5,A6,NDIM,NKP,NDK,
     .  NSTEP,IVARY)

        IF (J1 .NE. 0) J1=3
        IF (J2 .NE. 0) J2=NPROJ
        IF (J3 .NE. 0) J3=NPROJF
        IF (J4 .NE. 0) J4=NPROJB
        JJ=J1+J2+J3+J4
C
C 30.03.92/mbs
C
        IF (JJ .NE. 0) THEN
          WRITE(7,310) J1,J2,J3,J4,NMESH
  310     FORMAT(5I4)
          WRITE(7,320) (NCARD(I),I=1,MXCARD)
          WRITE(7,320) (NCARDF(I),I=1,MXCARD)
          WRITE(7,320) (NCARDB(I),I=1,MXCARD)
  320     FORMAT(5I4)
          WRITE(7,330) ((TITL(I,J),J=1,MXTITL),I=1,MXCARD), 
     .                 ((TITLF(I,J),J=1,MXTITL),I=1,MXCARD),
     .                 ((TITLB(I,J),J=1,MXTITL),I=1,MXCARD)
  330     FORMAT(A80)
        ENDIF

        IF (JJ .NE. 0) CALL SMOOTH(ENDK,W,B1,B2,B3,B4,XNORM,ID,NDK)
        IF (J2 .NE. 0) CALL SMOOTO(D,A,B1,B2,B3,NDIM,NKP,XNORM,IPROJ,
     .                             NDPR,NPROJ,NDK,2)
        IF (J3 .NE. 0) CALL SMOOTO(D,A,B1,B2,B3,NDIM,NKP,XNORM,IPROJF,
     .                             NDPRF,NPROJF,NDK,4)
        IF (J4 .NE. 0) CALL SMOOTB(D,B1,B2,B3,XNORM,NDIM,NKP,NDK)
  340 CONTINUE
      RETURN
      END
