C ********************************************************************
C * KMESHPEG - Program Package in Order to Compute k-sets            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  CRECT1                                                          *
C *                                                                  *
C *  Usage......: Generates k-point mesh of centered rectangular     *
C *               lattice, grid type 1, according to R. Ramirez,     *
C *               M.C. Boehm, Int. J. Quant. Chem. 30 (1986) 391     *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs)                              *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: 11.11.93                                           *
C *  Revision...: -                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C ********************************************************************

      PROGRAM CRECT1

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      CHARACTER*80 STRING

      WRITE(6,10)
10    FORMAT(/,' Generate k-point mesh (centered rectangular lattice,',
     .       ' type 1)',/)
20    WRITE(6,30)
30    FORMAT(' Number of k-points along x direction, total is ',
     .       '(N^2+1)/2: ',$)
      READ(5,40,ERR=20) N
40    FORMAT(I3)
      IF (N .LT. 3) GOTO 20
      IF ((2*(N/2)) .EQ. N) GOTO 20

      WRITE(6,50)
50    FORMAT(' Enter optional string for output options: ',$)
      READ(5,60) STRING
60    FORMAT(A80)

      WRITE(1,65) (N*N+1)/2
65    FORMAT('!',/,
     .       '! Centered rectangular lattice, grid type 1, ',I3,
     .       ' k-points,',/,
     .       '! according to R. Ramirez, M.C. Boehm, Int. J. Quant. '
     .       'Chem. 30 (1986), 391',/,'!')
      X=0.0D0   
      Y=0.5D0/DBLE(N-1)

      NHALF=(N+1)/2
      DO 100 I=1,N        
        A=X+Y*DBLE(I-1)
        DO 90 J=1,NHALF
          B=X+Y*DBLE(J-1)
       
          IF ((J .EQ. 1) .AND. ((I .EQ. 1) .OR. (I .EQ. N))) THEN 
C
C GAMMA, X point (2mm).
C
            NWEIGT=1
          ELSE IF ((I .EQ. 1) .OR. (J .EQ. 1)) THEN
C
C Y point,  DELTA and SIGMA line (m).
C
            NWEIGT=2
          ELSE IF (I .EQ. N) THEN
C
C D line (m).
C
            NWEIGT=2
          ELSE IF ((I .EQ. NHALF) .AND. (J .EQ. NHALF)) THEN
C
C C' point (2).
C
            NWEIGT=2
          ELSE
C
C General point (1).
C
            NWEIGT=4
          ENDIF

          IF ((J .EQ. NHALF) .AND. (I .GT. NHALF)) THEN
          ELSE
            WRITE(1,70) A,B,0.0D0,NWEIGT
70          FORMAT('KPIN ',3F13.10,I2)
            IF (STRING(1:1) .GT. ' ') THEN
              WRITE(1,80) STRING
80            FORMAT(A80)
            ENDIF
          ENDIF
90      CONTINUE
100   CONTINUE
      STOP
      END
