C ********************************************************************
C * TRANSC - Transformation of Cartesian to Fractional Coordinates   *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  TRANSC                                                          *
C *                                                                  *
C *  Usage......: Transform cartesian into fractional coordinates    *
C *               and vice versa. Lit.: J.D. Dunitz, X-Ray analysis  *
C *               and the structure of organic molecules, Cornell    *
C *               University 1979, p. 235-238                        *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs), University of Berne, Inst.  *
C *               of Inorg. and Phys. Chemistry, Freiestr. 3,        *
C *               CH-3012 Berne                                      *
C *  Date.......: 11.09.92                                           *
C *  Revision...: 11.05.93/mbs                                       *
C *  Remarks....: Look for "AIX","VMS" or "MS-DOS".                  *
C *                                                                  *
C *  Calls......: READL,READA,MATMUL                                 *
C *                                                                  *
C ********************************************************************

      PROGRAM TRANSC

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      DIMENSION    TRANSM(3,3),XIN(3,1),XOUT(3,1),ORIGIN(3),AVEC(3),
     .             BVEC(3),XTEMP(3,1),VTEMP(3,1),ROTMAT(3,3),
     .             ROTMT2(3,3)
      CHARACTER*10 ATNAME
      LOGICAL      TOFRAC

      COMMON /IOUNIT/IIN,IOUT,ISTART(40)

      COMMON /KEYWRD/SLINE
      CHARACTER SLINE*80

      PARAMETER(PI=3.14159265358979D0,RAD=PI/180.D0)

      DATA ORIGIN/3*0.0D0/

      IIN=1
      IOUT=2

      WRITE(6,10)
   10 FORMAT(//,' *************************************************',/,
     .          ' *                     TRANSC                    *',/,
     .          ' *             Release September 1997            *',/,
     .          ' * --------------------------------------------- *',/,
     .          ' *    Transformation of Cartesian Coordinates    *',/,
     .          ' *    to Fractional Coordinates and vice versa   *',/,
     .          ' *                                               *',/,
     .          ' *                  M. Braendle                  *',/,
     .          ' *    Department of Chemistry and Biochemistry   *',/,
     .          ' *        University of Bern, Switzerland        *',/,
     .          ' *                                               *',/,
     .          ' *          (C) 1997 by Gion Calzaferri          *',/,
     .          ' *   TRANSC comes with absolutely no warranty.   *',/,
     .          ' * This is free software and you are welcome to  *',/,
     .          ' *   redistribute it under certain conditions.   *',/,
     .          ' *  For details see GNU General Public License.  *',/,
     .          ' *************************************************',/,
     .2X)

C
C See if inputfile exists.
C
C MS-DOS version: add FILE=' ' to OPEN statements. The program will then
C                 ask for the file names
C AIX or VMS version: Remove FILE=' ' from OPEN statements.
C
      OPEN(UNIT=IIN,STATUS='OLD',FILE=' ',ERR=300)
      OPEN(UNIT=IOUT,STATUS='NEW',FILE=' ')
C      OPEN(UNIT=IIN,STATUS='OLD',ERR=300)
C      OPEN(UNIT=IOUT,STATUS='NEW')
      TOFRAC=.FALSE.
C
C Format of input file:
C
C CELL a b c alpha beta gamma
C ATOM or POS atomname coordinate1 coordinate2 coordinate3
C .
C .
C
C All other lines not containing the above keywords will be read, but
C not handled.
C

C
C Read cell dimensions and get type of transformation:
C If a=b=c=1.0, alpha=beta=gamma=90.0 transformation from cartesian to
C fractional coordinates is assumed.
C
20    CALL READL
      IF (INDEX(SLINE,'END') .NE. 0) THEN
        WRITE(6,30)
30      FORMAT(/,' No cell parameters detected. Check if keyword CEll ',
     .         ' exists.')
        CLOSE(1)
        CLOSE(2)
        STOP
      ELSE IF (INDEX(SLINE,'CE') .NE. 0) THEN
        A=READA(SLINE,ISTART(2),80)
        B=READA(SLINE,ISTART(3),80)
        C=READA(SLINE,ISTART(4),80)
        ALPHA=READA(SLINE,ISTART(5),80)
        BETA= READA(SLINE,ISTART(6),80)
        GAMMA=READA(SLINE,ISTART(7),80)
      ELSE
        WRITE(IOUT,40) SLINE
40      FORMAT(A80)
        GOTO 20
      ENDIF

      IF ((A .EQ. 1.0D0) .AND. (B .EQ. 1.0D0) .AND. (C .EQ. 1.0D0) .AND.
     .    (ALPHA .EQ. 90.0D0) .AND. (BETA .EQ. 90.0D0) .AND.
     .    (GAMMA .EQ. 90.0D0)) THEN
C
C Transformation of cartesian into fractional coordinates.
C
        TOFRAC=.TRUE.
        WRITE(6,50)
50      FORMAT(/,' Transformation from cartesian into fractional ',
     .         'coordinates:',//,
     .         ' Enter unit cell dimensions:',/)
60      WRITE(6,70)
70      FORMAT(' a    : ',$)
        READ(5,180,ERR=60) A
80      WRITE(6,90)
90      FORMAT(' b    : ',$)
        READ(5,180,ERR=80) B
100     WRITE(6,110)
110     FORMAT(' c    : ',$)
        READ(5,180,ERR=100) C
120     WRITE(6,130)
130     FORMAT(' alpha: ',$)
        READ(5,180,ERR=120) ALPHA
140     WRITE(6,150)
150     FORMAT(' beta : ',$)
        READ(5,180,ERR=140) BETA
160     WRITE(6,170)
170     FORMAT(' gamma: ',$)
        READ(5,180,ERR=160) GAMMA
180     FORMAT(F12.7)

        WRITE(6,190)
190     FORMAT(/,'                                         ----x---- ',
     .         '----y---- ----z----')
200     WRITE(6,210)
210     FORMAT(' Origin of fractional coordinate system: ',$)
        READ(5,220,ERR=200) ORIGIN(1),ORIGIN(2),ORIGIN(3)
220     FORMAT(3F10.5)

        WRITE(6,230)
230     FORMAT(/,'                         ----x---- ----y----',
     .         ' ----z----')
240     WRITE(6,245)
245     FORMAT(' Components of a-vector: ',$)
        READ(5,220,ERR=240) AVEC(1),AVEC(2),AVEC(3)

        IF ((AVEC(1) .EQ. 0.0D0) .AND. (AVEC(2) .EQ. 0.0D0) .AND.
     .      (AVEC(3) .EQ. 0.0D0)) THEN
          AVEC(1)=1.0D0
          AVEC(2)=0.0D0
          AVEC(3)=0.0D0
        ENDIF

        ALPHA=ALPHA*RAD
        BETA=BETA*RAD
        GAMMA=GAMMA*RAD

        COSALP=COS(ALPHA)
        COSBET=COS(BETA)
        COSGAM=COS(GAMMA)
        SINGAM=SIN(GAMMA)

246     WRITE(6,247)
247     FORMAT(' Components of b-vector: ',$)
        READ(5,220,ERR=246) BVEC(1),BVEC(2),BVEC(3)

        IF ((BVEC(1) .EQ. 0.0D0) .AND. (BVEC(2) .EQ. 0.0D0) .AND.
     .      (BVEC(3) .EQ. 0.0D0) .AND. (AVEC(3) .EQ. 0.0D0)) THEN
          BVEC(1)=COSGAM*AVEC(1)-SINGAM*AVEC(2)
          BVEC(2)=SINGAM*AVEC(1)+COSGAM*AVEC(2)
          BVEC(3)=0.0D0
        ENDIF

        V=1.0D0-COSALP*COSALP-COSBET*COSBET-COSGAM*COSGAM+
     .    2.0D0*COSALP*COSBET*COSGAM
        V=SQRT(V)

        ALPHA=ALPHA/RAD
        BETA=BETA/RAD
        GAMMA=GAMMA/RAD
C
C Evaluate rotation angles and normal vectors in order to align later
C the x and a vector and put b vector into (x,y)-plane.
C
        R1=SQRT(AVEC(1)*AVEC(1)+AVEC(2)*AVEC(2)+AVEC(3)*AVEC(3))
        R2=SQRT(AVEC(2)*AVEC(2)+AVEC(3)*AVEC(3))

        COSPHI=AVEC(1)/R1
        SINPHI=SIN(ACOS(COSPHI))

        IF (R2 .EQ. 0.0D0) THEN
          D1=0.0D0
          D2=1.0D0
          D3=0.0D0
        ELSE
          D1=0.0D0
          D2=AVEC(3)/R2
          D3=-AVEC(2)/R2
        ENDIF

        ROTMAT(1,1)=D1*D1+(D2*D2+D3*D3)*COSPHI
        ROTMAT(1,2)=D1*D2*(1.0D0-COSPHI)-D3*SINPHI
        ROTMAT(1,3)=D1*D3*(1.0D0-COSPHI)+D2*SINPHI
        ROTMAT(2,1)=D1*D2*(1.0D0-COSPHI)+D3*SINPHI
        ROTMAT(2,2)=D2*D2+(D1*D1+D3*D3)*COSPHI
        ROTMAT(2,3)=D2*D3*(1.0D0-COSPHI)-D1*SINPHI
        ROTMAT(3,1)=D1*D3*(1.0D0-COSPHI)-D2*SINPHI
        ROTMAT(3,2)=D2*D3*(1.0D0-COSPHI)+D1*SINPHI
        ROTMAT(3,3)=D3*D3+(D1*D1+D2*D2)*COSPHI

        CALL MATMUL(ROTMAT,BVEC,VTEMP,3,3,3,1)

        R3=SQRT(VTEMP(2,1)*VTEMP(2,1)+VTEMP(3,1)*VTEMP(3,1))
        IF (R3 .EQ. 0.0D0) THEN
          COSPSI=1.0D0
          SINPSI=0.0D0
        ELSE
          SINPSI=VTEMP(3,1)/R3
          COSPSI=COS(ASIN(SINPSI))
        ENDIF
        IF (VTEMP(2,1) .LT. 0.0D0) THEN
          COSPSI=-COSPSI
          SINPSI=-SINPSI
        ENDIF

        ROTMT2(1,1)=1.0D0
        ROTMT2(1,2)=0.0D0
        ROTMT2(1,3)=0.0D0
        ROTMT2(2,1)=0.0D0
        ROTMT2(2,2)=COSPSI
        ROTMT2(2,3)=-SINPSI
        ROTMT2(3,1)=0.0D0
        ROTMT2(3,2)=SINPSI
        ROTMT2(3,3)=COSPSI
C
C Transformation matrix.
C
        TRANSM(1,1)=1.0D0/A
        TRANSM(1,2)=-COSGAM/(A*SINGAM)
        TRANSM(1,3)=(COSGAM*COSALP-COSBET)/(A*V*SINGAM)
        TRANSM(2,1)=0.0D0
        TRANSM(2,2)=1.0D0/(B*SINGAM)
        TRANSM(2,3)=(COSGAM*COSBET-COSALP)/(B*V*SINGAM)
        TRANSM(3,1)=0.0D0
        TRANSM(3,2)=0.0D0
        TRANSM(3,3)=SINGAM/(C*V)
      ELSE
C
C Transformation of fractional into cartesian coordinates.
C
        ALPHA=ALPHA*RAD
        BETA=BETA*RAD
        GAMMA=GAMMA*RAD

        COSALP=COS(ALPHA)
        COSBET=COS(BETA)
        COSGAM=COS(GAMMA)
        SINGAM=SIN(GAMMA)

        V=1.0D0-COSALP*COSALP-COSBET*COSBET-COSGAM*COSGAM+
     .    2.0D0*COSALP*COSBET*COSGAM
        V=SQRT(V)

        TRANSM(1,1)=A
        TRANSM(1,2)=B*COSGAM
        TRANSM(1,3)=C*COSBET
        TRANSM(2,1)=0.0D0
        TRANSM(2,2)=B*SINGAM
        TRANSM(2,3)=C*(COSALP-COSBET*COSGAM)/SINGAM
        TRANSM(3,1)=0.0D0
        TRANSM(3,2)=0.0D0
        TRANSM(3,3)=C*V/SINGAM

        A=1.0D0
        B=A
        C=A
        ALPHA=90.0D0
        BETA=ALPHA
        GAMMA=ALPHA
      ENDIF
C
C Write cell parameters.
C
      WRITE(IOUT,250) A,B,C,ALPHA,BETA,GAMMA
250   FORMAT('CELL ',6F12.7)

260   CALL READL
      IF (INDEX(SLINE,'END') .NE. 0) THEN
        IDUMMY=INT(READA(SLINE,ISTART(2),80))
        WRITE(IOUT,290) SLINE
        IF (IDUMMY .GT. 0) THEN
          GOTO 260
        ELSE
          GOTO 310
        ENDIF
      ELSE IF ((INDEX(SLINE,'AT') .NE. 0) .OR.
     .         (INDEX(SLINE,'POS') .NE. 0)) THEN
        ATNAME=SLINE(ISTART(2):ISTART(3)-1)
        XIN(1,1)=READA(SLINE,ISTART(3),80)
        XIN(2,1)=READA(SLINE,ISTART(4),80)
        XIN(3,1)=READA(SLINE,ISTART(5),80)
        IF (TOFRAC) THEN
          DO 270 I=1,3
            XTEMP(I,1)=XIN(I,1)-ORIGIN(I)
270       CONTINUE
          CALL MATMUL(ROTMAT,XTEMP,VTEMP,3,3,3,1)
          CALL MATMUL(ROTMT2,VTEMP,XIN,3,3,3,1)
        ENDIF
        CALL MATMUL(TRANSM,XIN,XOUT,3,3,3,1)
        WRITE(IOUT,280) ATNAME,XOUT(1,1),XOUT(2,1),XOUT(3,1)
280     FORMAT('ATOM ',A10,3F12.7)
        GOTO 260
      ELSE
        WRITE(IOUT,290) SLINE
290     FORMAT(A80)
        GOTO 260
      ENDIF

300   WRITE(6,305)
305   FORMAT(' Inputfile could not be opened.')

310   CLOSE(1)
      CLOSE(2)
      STOP
      END
